/*
 ******************************************************************************
 ******************************************************************************
 *****                             PEXtk                                  *****
 *****									  *****
 *****		    An Immediate Mode Interface to PEX			  *****
 *****									  *****
 *****				Test Programs				  *****
 *****									  *****
 *                                                                            *
 *	Copyright 1991, 1993. All Rights Reserved                             *
 *			SHOgraphics					      *
 *			321 Soquel Way           			      *
 *			Sunnyvale, CA      94086  			      *
 *                                                                            *
 *****									  *****

    Permission to use, copy, modify, and distribute this software and
 its documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in
 supporting documentation, and that the name of ShoGraphics not be
 used in advertising or publicity pertaining to distribution of the 
 software without specific, written prior permission.

 SHOGRAPHICS DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 INCLUDING ALL EXPRESSED AND IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS. IN NO EVENT SHALL SHOGRAPHICS BE LIABLE FOR ANY DAMAGES OF ANY
 KIND, INCLUDING BUT NOT LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL 
 DAMAGES RESULTING FROM LOSS OF USE, DATA OR PROSPECTIVE PROFITS, WHETHER 
 IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, ARISING OUT 
 OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

 ************************************************************************ 
 *		PEXtk Test Program for Sphere routine
 */
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <stdio.h>
#include <PEXtk.h>

    Fcoord3D	 	ctr;
    int			iter, view_type;
    int			pause_flag;
    float		near, far;
    float		left, right, bottom, top, fov, aspect;
    int			win_width, win_height;


main(argc, argv)
int argc; char **argv;
{
    int i,j,k,m,n;
    Display 		*dpy;
    Window 		window;
    Fcoord3D	 	vt, vn, vtt[255];
    PEXtk_LIGHT		lgt;
    PCOLOR		colorObj;
    int		 	*infoData;
    int			vpts[4][4];

    view_type = 0; pause_flag = 0;
    iter = 100;
    win_width=500; win_height=500;
    left = -6.0; right = 6.0; bottom = -6.0; top = 6.0;
    near = 1.0; far = 1000.0; aspect = 1.0; fov = 90.0;

    parse_line(argc,argv);

    if ((dpy = XOpenDisplay ("")) == NULL) {
	printf(" Error opening display\n");
	return 1;
    }

    if(Pinit(dpy)){
	printf("ERROR... initializing PEX server\n");
	exit(1);
    }

    if( (window = Pcreate_window(dpy, win_width, win_height)) == NULL){
	printf("ERROR... creating window\n");
	exit(1);
    }

    Pset_drawable( window );
    Pzbuffer(1);
    Pbuffer_mode(  PEXtk_DOUBLE_BUFFER );
    Pviewport( 0, win_width-1, 0, win_height-1);
    Pbackface( PEXtk_CULL_BACKFACE);

    colorObj.color.rgb.red   = 0.9; colorObj.color.rgb.green = 0.0;
    colorObj.color.rgb.blue  = 0.4;

	Set_view();
    Pshade_mode( PEXtk_SHADE_LIGHTS|PEXtk_SHADE_SPECULAR|PEXtk_SHADE_COLORS);

    Plight_color_rgb( 0.5, 0.5, 0.5);
    Plight_ambient( .6);
    lgt.type = PEXtk_LIGHT_INFINITE;
    lgt.dir.x = 0.0; lgt.dir.y = 0.0; lgt.dir.z = -1.0;
    lgt.pos.x = 20.0; lgt.pos.y = 20.0; lgt.pos.z = 21.0;
    lgt.atten1 = .5;
    lgt.atten2 = 0.0;
    Plight( 1, &lgt);
    Pspecular_color_rgb( 1.0, 1.0, 1.0);	/* white specular */

	Pcolor( &colorObj);
	Pline_color( &colorObj);
	Ptext_color_rgb( 0.0, 0.0, 1.0);

	Pset_precision( 20, 20);

	Create_str();
    i = 0;
    while( i < iter ){
	Pclear_rgb( 0.0, 0.0, 0.0);
	Pload_identity();
	Ptranslate( 0.0, 0.0, -2.0, 1);
	Protate_z( i*2.0, 1 );
	Protate_x( i*1.0, 1 );
	Pcall_struct( 1);
	Pswap_buffers(); 
	i++;
    }
    if( pause_flag ){
	printf("enter a char "); fscanf( stdin, "%c",&i);
	}
    if(Pdelete_struct(1))
	printf("ERROR... deleting structure 1\n");
    Pexit();
    XDestroyWindow( dpy, window);
    XCloseDisplay( dpy);
    exit(0);
}

Set_view()
{
    int i, j;
    switch( view_type ){
	case 0:
	    Ppersp_view( 0, fov, aspect, near, far);
	    break;
	case 1:
	    Portho_view( 0, left, right, bottom, top, near, far);
	    break;
	case 2:
	    Portho_2d_view( 0, left, right, bottom, top );
	    break;
	case 3:
	    Pwindow_view( 0, left, right, bottom, top, near, far );
	    break;
	default:
	    Ppersp_view( 0, fov, aspect, near, far);
	    break;
    }
}

Create_str()
{
    int i, j, k, m, n;
    Fcoord3D	 vn, vt[255];
    char buf[20];

    Pcreate_struct(1);
	Psphere_fill( 0.0, 0.0, 0.0, 1.0);
    Pclose_struct();
}

/*
 *                      +---------------------------+
 * ---------------------+         parse_line        +------------------------
 *                      +---------------------------+
 *  FUNCTION:
 *      Parses the command line.
 *
 *  USAGE:
 *      parse_line(argc, argv,options)
 *	int	   argc 	   (in)  - number of arg's
 *	char   	   argv 	   (in)  - pointer to arg list
 *
 *  DESCRIPTION:
 *	This routine will parse the command line for the following options:
 *	   opt  argument	type	  comment
 *	   ---  ------------    ----      ----------------------------------
 *	    -f filename.obj	string	  .obj file to read
 *	    -i iter		int	  number of iterations
 *	    -c 			void	  sets clip flag
 *						clip rect will be 100 x pixels
 *						and 50 y pixels less than
 *						viewport size
 *	    -p 			void	  sets pause flag
 *	    -m view		int	  view: 0=persp,1=ortho,2=ortho2,
 *					  3=window
 *	    -w size		int	  window size
 *	    -z trans		float	  z translation value
 *	    -h near		float	  near (hither) plane
 *	    -y far		float	  far (yon) plane
 *	    -a aspect		float	  aspect ratio
 *	    -l left		float	  left
 *	    -r right		float	  right
 *	    -t top		float	  top
 *	    -b bottom		float	  bottom
 *                                
*/
int parse_line(argc,argv)
int argc;
char **argv;
{
     char *temp,ffile[32];
     extern char *optarg;
     extern int optind;
     int file_err;
     int c;


    file_err = 1;

    while((c=getopt(argc,argv,"a:i:h:y:l:r:b:t:m:f:t:w:cpvz:")) != EOF)
    {
	switch(c){
	    case 'i':
		  sscanf(optarg,"%d",&iter);
		break;
	    case 'm':
		  sscanf(optarg,"%d",&view_type);
		break;
	    case 'a':
		  sscanf(optarg,"%f",&aspect);
		break;
	    case 'p':
		  pause_flag = 1;
		break;
	    case 'w':
		  sscanf(optarg,"%d",&win_width);
		  win_height = win_width;
		break;
	    case 'h':
		  sscanf(optarg,"%f",&near);
		break;
	    case 'y':
		  sscanf(optarg,"%f",&far);
		break;
	    case 'l':
		  sscanf(optarg,"%f",&left);
		break;
	    case 'r':
		  sscanf(optarg,"%f",&right);
		break;
	    case 'b':
		  sscanf(optarg,"%f",&bottom);
		break;
	    case 't':
		  sscanf(optarg,"%f",&top);
		break;
	    default:
		Usage();
		break;
	}
    }

}

Usage()
{
    printf("Error...  Usage is:\n");
    printf(" vue_test [options] file.obj\n");
    printf(" options: \n");
    printf("\t -m view {0-3}\t(view type)\n");
    printf("\t -i iter\t(number of iterations)\n");
    printf("\t -a aspect\t(aspect ratio, float)\n");
    printf("\t -l left\t(left coord, float)\n");
    printf("\t -r right\t(right coord, float)\n");
    printf("\t -t top\t\t(top coord, float)\n");
    printf("\t -b bottom\t(bottom coord, float)\n");
    printf("\t -w size\t(window size, int)\n");
    printf("\t -h hither\t(set hither plane, float)\n");
    printf("\t -y yon\t\t(set yon plane, float)\n");
    printf("\t -z z_value\n");
    printf("\t -p\t\t(pause flag)\n");
    printf("\t -f file.obj\n");
}
