/*
 ******************************************************************************
 ******************************************************************************
 *****                             PEXtk                                  *****
 *****									  *****
 *****		    An Immediate Mode Interface to PEX			  *****
 *****									  *****
 *****									  *****
 *                                                                            *
 *	Copyright 1991, 1993. All Rights Reserved                             *
 *			SHOgraphics					      *
 *			321 Soquel Way           			      *
 *			Sunnyvale, CA      94086  			      *
 *                                                                            *
 *****									  *****

    Permission to use, copy, modify, and distribute this software and
 its documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in
 supporting documentation, and that the name of SHOgraphics not be
 used in advertising or publicity pertaining to distribution of the 
 software without specific, written prior permission.

 SHOGRAPHICS DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 INCLUDING ALL EXPRESSED AND IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS. IN NO EVENT SHALL SHOGRAPHICS BE LIABLE FOR ANY DAMAGES OF ANY
 KIND, INCLUDING BUT NOT LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL 
 DAMAGES RESULTING FROM LOSS OF USE, DATA OR PROSPECTIVE PROFITS, WHETHER 
 IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, ARISING OUT 
 OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

 ************************************************************************ 
 *		PEXtk Test Program for begin/end routines
 *			and the Ppoly_point routines
 */
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <stdio.h>
#include <PEXtk.h>

#include "read_demo_obj.c"

    Fcoord3D	 	ctr;
    int			iter, verbose_flag, ModelType, color_flag;
    int			poly_norm_flag, i_style, str_flag, pause_flag;
    int 		win_width, win_height;
    PCOLOR		colorObj, colorList[256];

    typedef struct BLD_TEST {
	int	(*begin)();
	int	(*end)();
    } BLD_TEST;

    struct BLD_OBJECT {
	BLD_TEST	funcs[8];
    } bld_object;

main(argc, argv)
int argc; char **argv;
{
    int 		i,j,k,m,n;
    Display 		*dpy;
    Window 		window;
    Fcoord3D	 	vt, vn, vtt[255], fn;
    float 		rinc;
    int 		End_default();

    ModelType = 0; poly_norm_flag = verbose_flag = color_flag = 0;
    iter = 100; i_style = PEXtk_FILL_HOLLOW; pause_flag = str_flag = 0;
    win_width=500; win_height=500;

    parse_line(argc,argv);

    bld_object.funcs[0].begin = (int (*)()) Pbegin_poly;
    bld_object.funcs[0].end   = (int (*)()) Pend_poly;
    bld_object.funcs[1].begin = (int (*)()) Pbegin_line;
    bld_object.funcs[1].end   = (int (*)()) Pend_line;
    bld_object.funcs[2].begin = (int (*)()) Pbegin_tmesh;
    bld_object.funcs[2].end   = (int (*)()) Pend_tmesh;
    bld_object.funcs[3].begin = (int (*)()) Ppoly_point_2d;
    bld_object.funcs[3].end   = (int (*)()) Ppoly_close;
    bld_object.funcs[4].begin = (int (*)()) Ppoly_point_3d;
    bld_object.funcs[4].end   = (int (*)()) Ppoly_close;
    bld_object.funcs[5].begin = (int (*)()) Ppoly_point_nv;
    bld_object.funcs[5].end   = (int (*)()) Ppoly_close;
    bld_object.funcs[6].begin = (int (*)()) Ppoly_point_uv;
    bld_object.funcs[6].end   = (int (*)()) Ppoly_close;
    bld_object.funcs[7].begin = (int (*)()) Ppoly_point_nv_uv;
    bld_object.funcs[7].end   = (int (*)()) Ppoly_close;

    if ((dpy = XOpenDisplay ("")) == NULL) {
	printf(" Error opening display\n");
	return 1;
    }

    if(Pinit(dpy)){
	printf("ERROR... initializing PEX server\n");
	exit(1);
    }

    if( (window = Pcreate_window(dpy, win_width, win_height)) == NULL){
	printf("ERROR... creating window\n");
	exit(1);
    }

    Pset_drawable( window );
    Pbuffer_mode(  PEXtk_DOUBLE_BUFFER );
    Pbackface( PEXtk_CULL_NONE);
    Pviewport( 0, win_width-1, 0, win_height-1);
    Ppersp_view( 0, 90.0, 1.0, 1.0, 1000.0);

    colorObj.color.rgb.red   = 0.9; colorObj.color.rgb.green = 0.0;
    colorObj.color.rgb.blue  = 0.4;

    colorList[0].color.rgb.red   = 1.0; colorList[0].color.rgb.green = 0.0;
    colorList[0].color.rgb.blue  = 0.0;
    colorList[1].color.rgb.red   = 0.0; colorList[1].color.rgb.green = 1.0;
    colorList[1].color.rgb.blue  = 0.0;
    colorList[2].color.rgb.red   = 0.0; colorList[2].color.rgb.green = 0.0;
    colorList[2].color.rgb.blue  = 1.0;
    rinc = 1.0/(256-3);
	colorList[3].color.rgb.red   = 1.0;
	colorList[3].color.rgb.green = 0.0;
	colorList[3].color.rgb.blue  = 0.0;
    for(i=3;i<256;i++){
	colorList[i].color.rgb.red   = colorList[3].color.rgb.red - rinc*i;
	colorList[i].color.rgb.green = colorList[3].color.rgb.green + rinc*i;
	colorList[i].color.rgb.blue  = 0.0;
    }

    if( ModelType < 3 )
	Create_Build_Struct(str_flag);
    else
	Create_Ppoly_Struct(str_flag);

	Pcolor( &colorObj);
	Pline_color( &colorObj);
	Ptranslate( -ctr.x, -ctr.y, 0.0, 1);
    i = 0;
    while( i < iter ){
	Pclear_rgb( 0.0, 0.0, 0.0);
	Protate_z( 3.0, 1 );
	if( str_flag )
	    Pcall_struct( 1);
	else {
	    if( ModelType < 3 )
		Create_Build_Struct(str_flag);
	    else
		Create_Ppoly_Struct(str_flag);
	}

	Pswap_buffers(); 
	i++;
    }
    if( pause_flag ){
	printf("enter a char ");
	fscanf( stdin, "%c",&i);
	}
    Pexit();
    XDestroyWindow( dpy, window);
    XCloseDisplay( dpy);
    exit(0);
} 

End_default()
{
}

Create_Build_Struct(num)
int num;
{
    int i,j,k,l,m,n;
    Fcoord3D	 vt, vn, vtt[255], fn;

    if( num != 0 )
    Pcreate_struct(1);
	for(i=0,j=0; i<poly_cnt; i++)
	{
	   k = vert_indices[j];
	   (*bld_object.funcs[ModelType].begin)(i_style); /* Pbegin_.. */
	   if( poly_norm_flag ){
	       Calc_Facet_Normal( (int) vert_indices[j],
			(int) vert_indices[j+1],(int)vert_indices[j+2], &fn);
		Ppoly_normal( fn.x, fn.y, fn.z );
	   }
           for( m=0;m<facet_Count[i]; m++, j++){
		n = vert_indices[j];
		vt.x = verts[n].x; vt.y = verts[n].y; vt.z = verts[n].z;
	       if( color_flag )
		 Pvertex_color( &colorList[n%255]);
	       if( vnct ){
		 n = vn_indices[j];
		 if( (short) n != (short) -1 ){
		  vn.x = vertn[n].x; vn.y = vertn[n].y; vn.z = vertn[n].z;
		 }
		 Pvertex_normals( 1, &vn );
	       }
	     Pvertex( &vt);
	   }
	   if( ModelType == 1 ){ /* add one more vert for line type */
	     vt.x = verts[k].x; vt.y = verts[k].y; vt.z = verts[k].z;
	     Pvertex( &vt);
	       if( color_flag )
		 Pvertex_color( &colorList[k%255]);
	   }
	   (*bld_object.funcs[ModelType].end)();
	}
    if( num != 0 )
      Pclose_struct();
}

Create_Ppoly_Struct(num)
int num;
{
    int i,j,k,l,m,n;
    Fcoord3D	 vt, vn, vtt[255], fn;

    if( num != 0 )
    Pcreate_struct(1);

	for(i=0,j=0; i<poly_cnt; i++)
	{
	   k = vert_indices[j];
	   if( poly_norm_flag ){
	       Calc_Facet_Normal( (int) vert_indices[j],
			(int) vert_indices[j+1],(int)vert_indices[j+2], &fn);
		Ppoly_normal( fn.x, fn.y, fn.z );
	   }
           for( m=0;m<facet_Count[i]; m++, j++){
		n = vert_indices[j];
		vt.x = verts[n].x; vt.y = verts[n].y; vt.z = verts[n].z;
	       if( color_flag )
		 Pvertex_color( &colorList[m%256]);
		 vn.x = vn.y = 0.0; vn.z = 1.0;
	       if( ModelType == 5 || ModelType == 7 )
		  if( vnct ){
		    n = vn_indices[j];
		    if( n != -1 ){
		       vn.x = vertn[n].x; vn.y = vertn[n].y; vn.z = vertn[n].z;
		    }
	          } 
	    switch( ModelType ){
	      case 3:		/* Ppoly_point_2d */
	   	(*bld_object.funcs[ModelType].begin)( vt.x, vt.y);
	      break;
	      case 4:		/* Ppoly_point_3d */
	   	(*bld_object.funcs[ModelType].begin)( vt.x, vt.y, vt.z);
	      break;
	      case 5:		/* Ppoly_point_nv */
	   	(*bld_object.funcs[ModelType].begin)( vt.x, vt.y, vt.z,
		    vn.x, vn.y, vn.z);
	      break;
	      case 6:		/* Ppoly_point_uv */
	   	(*bld_object.funcs[ModelType].begin)( vt.x, vt.y, vt.z,
		    vt.x, vt.y, vt.z);
	      break;
	      case 7:		/* Ppoly_point_nv_uv */
	   	(*bld_object.funcs[ModelType].begin)( vt.x, vt.y, vt.z,
		    vn.x, vn.y, vn.z, vt.x, vt.y, vt.z);
	      break;
	    }
	   }	/* end vertex loop */
	   (*bld_object.funcs[ModelType].end)();
	}	/* end foreach poly */

    if( num != 0 )
      Pclose_struct();
}

/*
 *                      +---------------------------+
 * ---------------------+         parse_line        +------------------------
 *                      +---------------------------+
 *  FUNCTION:
 *      Parses the command line.
 *
 *  USAGE:
 *      parse_line(argc, argv,options)
 *	int	   argc 	   (in)  - number of arg's
 *	char   	   argv 	   (in)  - pointer to arg list
 *
 *  DESCRIPTION:
 *	This routine will parse the command line for the following options:
 *	   opt  argument	type	  comment
 *	   ---  ------------    ----      ----------------------------------
 *	    -c 			void	  sets color flag for colors per vertex
 *	    -f filename.obj	string	  .obj file to read
 *	    -i iter		int	  number of iterations
 *	    -r 			void	  sets retained structure mode
 *	    -s style		int	  interior style
 *	    -t type		int	  type of begin sequence: Choice of
 *						0=poly, 1=line, 2=tmesh
 *	    -v 			void	  sets verbose flag
 *	    -p 			void	  sets pause flag
 *	    -w size		int	  window size
 *	    -z trans		float	  z translation value
 *                                
*/
int parse_line(argc,argv)
int argc;
char **argv;
{
     char *temp,ffile[32];
     extern char *optarg;
     extern int optind;
     int file_err;
     int c;

    if( argc < 2 ){
	Usage();
	exit(1);
    }

    file_err = 1;

    while((c=getopt(argc,argv,"cnpri:f:s:t:vz:")) != EOF)
    {
	switch(c){
	    case 'c':
		  color_flag = 1;
		break;
	    case 'f':
		  read_demo_obj(optarg, &ctr, 1.0);
		  file_err = 0;
		break;
	    case 'i':
		  sscanf(optarg,"%d",&iter);
		break;
	    case 'n':
		  poly_norm_flag = 1;
		break;
	    case 'p':
		  pause_flag = 1;
		break;
	    case 'r':
		  str_flag = 1;
		break;
	    case 's':
		  sscanf(optarg,"%d",&i_style);
		  if( i_style != PEXtk_FILL_HOLLOW &&
		      i_style != PEXtk_FILL_SOLID )
			i_style = PEXtk_FILL_SOLID;
		break;
	    case 't':
		  sscanf(optarg, "%d", &ModelType);
		  /* Check for valid type */
		  if( ModelType < 0 || ModelType > 7 ){
		    printf("ERROR... type must be 0 <= type <= 7:, %d\n",
			ModelType);
		    ModelType = 0;
		  }
		break;
	    case 'v':
		  verbose_flag = 1;
		break;
	    case 'w':
		  sscanf(optarg,"%d",&win_width);
		  win_height = win_width;
		break;
	    case 'z':
		  sscanf(optarg,"%f",&z_trans_factor);
		break;
	}
    }

    for(; optind < argc;optind++ ){
		  read_demo_obj(argv[optind], &ctr, 1.0);
		  file_err = 0;
    }
    if( file_err ){
	Usage();
	exit(1);
    }
}

Usage()
{
    printf("Error...  Usage is:\n");
    printf(" bld_test -c -i iter -s style -t type -v -f file.obj\n");
    printf("   where:\n");
    printf("\t -t type {0-7}\t(primitive type)\n");
	printf("\t\t 0=polygon\n");
	printf("\t\t 1=polyline\n");
	printf("\t\t 2=trimesh\n");
	printf("\t\t 3=Ppoly_point_2d\n");
	printf("\t\t 4=Ppoly_point_3d\n");
	printf("\t\t 5=Ppoly_point_nv\n");
	printf("\t\t 6=Ppoly_point_uv\n");
	printf("\t\t 7=Ppoly_point_nv_uv\n");
    printf("\t -s style {1,2}\t(interior style)\n");
	printf("\t\t 1=Hollow\n");
	printf("\t\t 2=Solid\n");
    printf("\t -i iter\t(number of iterations)\n");
    printf("\t -c\t\t(color per vertex flag)\n");
    printf("\t -n\t\t(facet normal flag)\n");
    printf("\t -r\t\t(use retained structure mode)\n");
    printf("\t -p\t\t(pause flag)\n");
    printf("\t -w size\t(set width and height)\n");
    printf("\t -h hither\t(set hither plane)\n");
    printf("\t -y yon\t\t(set yon plane)\n");
    printf("\t -v\t\t(verbose flag)\n");
    printf("\t -f file.obj\n");
}

Calc_Facet_Normal(i,j,k,fn)
int i,j,k;
Fcoord3D *fn;
{
  double dx1,dx2,dy1,dy2,dz1,dz2;
  double xn,yn,zn;
  double fac;

  dx1 = verts[i].x - verts[j].x; dy1 = verts[i].y - verts[j].y;
  dz1 = verts[i].z - verts[j].z;
  dx2 = verts[j].x - verts[k].x; dy2 = verts[j].y - verts[k].y;
  dz2 = verts[j].z - verts[k].z;
  xn = (dy1*dz2) - (dz1*dy2);
  yn = (dz1*dx2) - (dx1*dz2);
  zn = (dx1*dy2) - (dy1*dx2);

  fac = sqrt( xn*xn + yn*yn + zn*zn);

  if(fac != 0.0){
	fac = 1.0 / fac;
	fn->x = xn*fac;
	fn->y = yn*fac;
	fn->z = zn*fac;
   }
}
