/*
 ******************************************************************************
 ******************************************************************************
 *****                     Open_X_Window.c                                *****
 *****									  *****
 *****	 	      X Window System Initialization			  *****
 *****									  *****
 *****									  *****
 *                                                                            *
 *	Copyright 1991, All Rights Reserved                                   *
 *			ShoGraphics					      *
 *			1890 North Shoreline Blvd			      *
 *			Mountain View, CA    94043			      *
 *                                                                            *
 *****									  *****

    Permission to use, copy, modify, and distribute this software and
 its documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in
 supporting documentation, and that the name of ShoGraphics not be
 used in advertising or publicity pertaining to distribution of the 
 software without specific, written prior permission.

 SHOGRAPHICS DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 INCLUDING ALL EXPRESSED AND IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS. IN NO EVENT SHALL SHOGRAPHICS BE LIABLE FOR ANY DAMAGES OF ANY
 KIND, INCLUDING BUT NOT LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL 
 DAMAGES RESULTING FROM LOSS OF USE, DATA OR PROSPECTIVE PROFITS, WHETHER 
 IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, ARISING OUT 
 OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

 ************************************************************************** */

#include <math.h>

float getgdesc();

#define pixelvalue(r,g,b) (unsigned long) \
 (((unsigned long)(r) << 16) | ((unsigned long)(g) << 8) | (unsigned long)(b) )

    GC gc;
    Display		 *dpy_id = NULL;
    Window		 window_id = NULL;
    XEvent 		 report;
    int                  win_screen;
    char                 *win_title = "PEXtk : Test Program";
    char                 *win_icon_name   = "PEXtk";
    Pixmap               win_icon_pixmap;
    XIconSize            **win_icon_list;
    XFontStruct 	 *xfont_info;
    XGCValues values;
    int			 win_window_width;
    int			 win_window_height;
    int                  win_icon_width = 32;
    int                  win_icon_height = 32;
    XClassHint           win_class_hints;
    XSizeHints           win_size_hints;   /* preferred window sizes */
    XWMHints             win_icon_hints;
    Visual               *win_visual;
    Colormap       	 win_colormap;
    XColor               win_fg;
    XColor               win_bg;
    XStandardColormap    rgb_info;
    XSetWindowAttributes win_atts;
    unsigned long        win_mask;
    unsigned int         win_class = InputOutput;
    int                  win_xpos = 0;                  /* window position */
    int                  win_ypos = 0;
    int                  win_depth;
    unsigned int         win_border_width = 4;
    unsigned long        pix_fg, pix_bg; 

	extern unsigned long lkup[256][3];
	extern unsigned long lkup_pix[256];
	static int cmap_size;

/*
 *  ROUTINE:	Open_X_Window
 *  PURPOSE:	Opens the Display and creates a window
 */
int
Open_X_Window(displayptr,window, win_width, win_height)
Display **displayptr;
Window  *window;
int         win_width, win_height;
{
    if( *displayptr == NULL )
	if ((*displayptr = XOpenDisplay ("")) == NULL) {
	    printf(" error opening display\n");return 1;
        }

    dpy_id 	 = *displayptr;		/* save data */
    win_window_width = win_width;
    win_window_height = win_height;

    win_screen   = DefaultScreen (*displayptr);
    win_colormap = DefaultColormapOfScreen(
			ScreenOfDisplay(*displayptr, win_screen));
 
    /*
     * This is a hack to make the HP work properly.
     * The low end HP 700's can not double buffer
     * the 24 bit visual, even if it has it.
     */
    win_depth = PEXtk_Get_Best_Visual( *displayptr, &win_visual);

    if( win_depth == 0 ){
	win_visual   = DefaultVisual (*displayptr, win_screen);
	win_depth    = DefaultDepth (*displayptr, win_screen);
    }

   /*
    * Create colormap
    */
#ifdef WHITEBG
    pix_bg = 0xffffff; /* White */
    pix_fg = 0x000000; /* Black */
    pix_bg = WhitePixel(dpy_id, win_screen); /* White */
    pix_fg = BlackPixel(dpy_id, win_screen); /* Black */
#else
  /* Use Black Background */
    pix_fg = WhitePixel(dpy_id, win_screen); /* White */
    pix_bg = BlackPixel(dpy_id, win_screen); /* Black */
#endif

    if(win_depth == 24)
    {
	/* printf(" Window depth = 24\n"); */
    	win_bg.red   = (pix_bg >> 16) << 8;
    	win_bg.green = (pix_bg & 0xff00);
    	win_bg.blue  = (pix_bg & 0xff) << 8;
    	win_bg.flags = DoRed | DoGreen | DoBlue;
    	win_fg.red   = (pix_fg >> 16) << 8;
    	win_fg.green = (pix_fg & 0xff00);
    	win_fg.blue  = (pix_fg & 0xff) << 8;
    	win_fg.flags = DoRed | DoGreen | DoBlue;

    if (XAllocColor (*displayptr, win_colormap, &win_bg))
        pix_bg = win_bg.pixel;

    if (XAllocColor (*displayptr, win_colormap, &win_fg))
        pix_fg = win_fg.pixel;
    } else {
    	win_bg.red   = 0;
    	win_bg.green = 0;
    	win_bg.blue  = 0;
    	win_bg.flags = DoRed | DoGreen | DoBlue;
    	win_fg.red   = 255 << 8;
    	win_fg.green = 255 << 8;
    	win_fg.blue  = 255 << 8;
    	win_fg.flags = DoRed | DoGreen | DoBlue;
   	if (XAllocColor (*displayptr, win_colormap, &win_bg))
        	pix_bg = win_bg.pixel;
    	else
        	pix_bg = 0;

    	if (XAllocColor (*displayptr, win_colormap, &win_fg))
        	pix_fg = win_fg.pixel;
    	else
        	pix_fg = 255;
    }
 

    win_mask     = CWBorderPixel | CWBackPixel | CWColormap | CWBackingStore;
    win_atts.backing_store    = WhenMapped;
    win_atts.colormap         = win_colormap;
    win_atts.background_pixel = pix_bg;
    win_atts.border_pixel     = pix_bg;

    *window = XCreateWindow
                       (*displayptr,
                        RootWindow (*displayptr, win_screen),
                        win_xpos, win_ypos,
                        win_width, win_height,
                        win_border_width,
                        win_depth,
                        win_class,
                        win_visual,
               		win_mask,
			&win_atts); 

    window_id = *window;

    win_size_hints.flags = USPosition | PPosition | PMinSize | PMaxSize;
    win_size_hints.x		 = 20;
    win_size_hints.y		 = 20;
    win_size_hints.max_width     = win_width;
    win_size_hints.max_height    = win_height;
    win_size_hints.min_width     = win_width;
    win_size_hints.min_height    = win_height;
	
    XStoreName      (*displayptr, *window, win_title);
    XSetClassHint   (*displayptr, *window, &win_class_hints);
    XSetNormalHints (*displayptr, *window, &win_size_hints);
	
   /*
    * Create resources
    */
    if ((xfont_info = XLoadQueryFont (*displayptr, "courier14")) == NULL)
     if ((xfont_info = XLoadQueryFont (*displayptr, "9x15")) == NULL)
      if ((xfont_info = XLoadQueryFont (*displayptr, "fixed")) == NULL)
        {
        printf("Error on XLoadQueryFont\n");
        exit(2);
        }

    gc = XCreateGC (*displayptr, *window, 0L, &values);

    XSetFont (*displayptr, gc, xfont_info->fid);
    XSetForeground (*displayptr, gc, pix_fg);
    XSetBackground (*displayptr, gc, pix_bg);
    XSetLineAttributes (*displayptr, gc, 0, LineSolid, CapButt, JoinMiter);
#ifdef FOO
    XSetGraphicsExposures(*displayptr, gc, False);
#endif

    XSelectInput(*displayptr, *window, ExposureMask);
    XMapWindow(*displayptr,*window);
    XWindowEvent(*displayptr, *window, ExposureMask, &report);

    XSelectInput (*displayptr, *window, ExposureMask | FocusChangeMask |
		OwnerGrabButtonMask | PointerMotionMask
		| KeyPressMask | ButtonPressMask | ButtonReleaseMask);

    return 0;
}

PEXtk_util_find_visual( dpy, win_visual)
Display *dpy;
Visual **win_visual;
{
    int	m_op, f_ev, f_err, i, j, ret_class, c_class;
    XVisualInfo vTemplate, *visualList;

    /* Check for the HPExtension to use HP type double-buffered
     * visuals
     */
    if(!XQueryExtension( dpy, "HPExtension",
                                &m_op, &f_ev, &f_err))
        return 0;
    /* Find the 12 bit visual
     */

    win_depth    = 0;
    /*
     *  See if any 12-bit visuals are available
     *  If there is a 12-bit, then look for a truecolor visual if
     *  it exists.
     */
        vTemplate.screen = DefaultScreen( dpy);
        vTemplate.depth  = 12;
        visualList = XGetVisualInfo( dpy, VisualScreenMask |
                        VisualDepthMask, &vTemplate, &i);
	c_class = 0;
        for(j=0;j<i;j++){
#if defined(__cplusplus) || defined(c_plusplus)
            ret_class = visualList[j].c_class;
#else
	    ret_class = visualList[j].class;
#endif
            if( ret_class > c_class ){
                *win_visual = visualList[j].visual;
                win_depth = 12;
		c_class = ret_class;
                /*
                 * Calculate the shift amount for each rgb mask
                 */
		if( c_class == TrueColor ){
		    /* free the visualinfo */
		    return 1;
		}
            }
        }
}

int
PGet_Queue()
{
        XCheckMaskEvent( dpy_id, ExposureMask | FocusChangeMask |
		OwnerGrabButtonMask | PointerMotionMask
		| KeyPressMask | ButtonPressMask | ButtonReleaseMask,
		&report);
        switch(report.type){
            case KeyPress:
		return 7;
            case ButtonPress:
		return 103;
	    default:
		break;
	}
	return 0;
}

PCreate_Window( str)
char *str;
{
    if( win_window_width == 0 )
	win_window_width  = 400;
    if( win_window_height == 0 )
	win_window_height  = 400;
    win_title = str;
    Open_X_Window( &dpy_id, &window_id, win_window_width, win_window_height);
    Pinit( dpy_id);
    Pset_drawable( window_id);
    Pzbuffer(1);
}

PSet_Window_Title(str)
char *str;
{
    win_title = str;
    if( dpy_id != NULL )
	XStoreName( dpy_id, window_id, win_title);
}

PSet_Window_Size( xsize, ysize, xpos, ypos)
int xsize, ysize, xpos, ypos;
{
    if( xsize != 0 )
	win_window_width  = xsize;
    if( ysize != 0 )
	win_window_height = ysize;
    win_xpos   = xpos;
    win_ypos   = ypos;
    if( dpy_id != NULL ){
	win_size_hints.flags = USPosition | PPosition | PMinSize | PMaxSize;
	win_size_hints.x	     = win_xpos;
	win_size_hints.y	     = win_xpos;
	win_size_hints.max_width     = win_window_width;
	win_size_hints.max_height    = win_window_height;
	win_size_hints.min_width     = win_window_width;
	win_size_hints.min_height    = win_window_height;
	
	XSetNormalHints( dpy_id, window_id, &win_size_hints);
    }
}

