/*
 ******************************************************************************
 ******************************************************************************
 *****                             PEXtk                                  *****
 *****									  *****
 *****		    An Immediate Mode Interface to PEX			  *****
 *****									  *****
 *****			    Pick Test Program    			  *****
 *****									  *****
 *                                                                            *
 *	Copyright 1993, All Rights Reserved                                   *
 *			SHOgraphics					      *
 *			321 Soquel Way 			      		      *
 *			Sunnyvale, CA  94086       		              *
 *                                                                            *
 *****									  *****

    Permission to use, copy, modify, and distribute this software and
 its documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in
 supporting documentation, and that the name of ShoGraphics not be
 used in advertising or publicity pertaining to distribution of the 
 software without specific, written prior permission.

 SHOGRAPHICS DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 INCLUDING ALL EXPRESSED AND IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS. IN NO EVENT SHALL SHOGRAPHICS BE LIABLE FOR ANY DAMAGES OF ANY
 KIND, INCLUDING BUT NOT LIMITED LIMITED TO SPECIAL, INDIRECT OR 
 CONSEQUENTIAL DAMAGES RESULTING FROM LOSS OF USE, DATA OR PROSPECTIVE
 PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS 
 ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
 SOFTWARE.

 ************************************************************************ 
 *		PEXtk Test Program for picking
 *
 *	This program will draw some text and markers at various locations,
 *	and also a polygon. It will then move a 'pick box' across the screen 
 *	to test picking.
 *	A vertex array is used to locate 3D graphics primitives randomly
 *	around the screen.
 *
 *	A 'parent' pick id of 100 is used for the text label at each vertex, 
 *	and each text item has a pick id of its vertex number + 1 (base 1).
 *	The text primitives are stored in a structure.
 *
 *	A 'parent' pick id of 200 is used for a marker at each vertex,
 *	and each marker item has a pick id of its vertex number + 201.
 *	The marker primitives are stored in a structure.
 *
 *	A 'parent' pick id of 300 is used for the polygon. The polygon has
 *	a pick id of 310. The polygon is stored in a structure.
 *
 *	For the default settings, the hit test should return:
 *		Got 2 hits
 *		  100   11
 *		  200   211
 *		Got 1 hits
 *		  300   310
 *		Got 3 hits
 *		  100   3
 *		  200   203
 *		  300   310
 *		Got 2 hits
 *		  100   12
 *		  200   212
 */
#include <stdio.h>

#include <X11/X.h>
#include <X11/Xlib.h>
#include <PEXtk.h>

#define NUM_VERTS	12
#define BUF_SIZE	1000
    int			incr, ViewType;
    int			pause_flag;
    int			vct, poly_cnt;
    float		near, far;
    float		left, right, bottom, top, fov, aspect;
    GC			gc;
    unsigned long	white_pix;
    int 		win_width=600, win_height=600;

Fcoord3D verts[NUM_VERTS];

main(argc, argv)
int argc; char **argv;
{
    int i,j,k,m,n, win_width=600, win_height=600;
    int x,y,xsize,ysize,max_hits,moreRet;
    int buffer[BUF_SIZE];
    Display 		 *dpy;
    Window 		 window;
    int		 	 *infoData;

    ViewType = 0; pause_flag = 0;
    incr = 100;
    left = -5.0; right = 5.0; bottom = -5.0; top = 5.0;
    near = 1.0; far = 1000.0; aspect = 1.0; fov = 90.0;

    parse_line(argc,argv);

    if ((dpy = XOpenDisplay ("")) == NULL) {
	printf(" Error opening display\n");
	return 1;
    }

    if(Pinit(dpy)){
	printf("ERROR... initializing PEX server\n");
	exit(1);
    }

    if( (window = Pcreate_window(dpy, win_width, win_height)) == NULL){
	printf("ERROR... creating window\n");
	exit(1);
    }

    init_data();
    vct = NUM_VERTS;
    gc = DefaultGC(dpy,DefaultScreen(dpy));
    white_pix = WhitePixel(dpy,DefaultScreen(dpy));

    Pset_drawable( window );
    Pzbuffer(1);

	/* setup defaults */
	Pbuffer_mode(  PEXtk_DOUBLE_BUFFER );
	Pviewport( 0, win_width-1, 0, win_height-1);
	Pbackface( PEXtk_CULL_BACKFACE);
	Ptext_size( .15, .15);
	Pcolor_rgb( 0.9, 0.0, 0.4);
	Pline_color_rgb( 0.9, 0.0, 0.4);
	Pmarker_type( PEXtk_MRKR_X);
	Pmarker_color_rgb( 0.0, 1.0, 0.0);

    Set_view();

    Create_Text(1);
    Create_Marker(2);
    Create_Poly(3);
    xsize = ysize = 100;
    max_hits = 10000;
    x = xsize/2; y = win_height / 2 - 80;

    while( x < win_width ){
	Pclear_rgb( 0.0, 0.0, 0.0);

	/* "Draw" the objects now to pick them */
        Pbegin_pick( x, y, xsize, ysize, max_hits );
	  Ppick_id(100);
	    Pexec_struct(1); /* draw text */
	  Ppick_id(200);
	    Pexec_struct(2); /* draw markers */
	  Ppick_id(300);
	    Pexec_struct(3); /* draw polygon */
        n = Pend_pick( buffer, BUF_SIZE, &moreRet);

	/* Draw the objects now to put on screen */
        Pexec_struct(1);
	Pexec_struct(2);
	Pexec_struct(3);
	/* Draw the 'pick box' */
	XSetForeground( dpy, gc, white_pix);
	XDrawRectangle(dpy, Pget_backbuffer(), gc, x-xsize/2,
        	win_height - y - ysize/2, xsize, ysize);
        Pswap_buffers();

	/* If there was any hits, then print the pick buffer contents */
	if( n != 0 ){
	    printf(" Got %d hits\n",n);
	    j = 0;
	    while(n > 0){
		k = buffer[j]; j++;
		for(m=0;m<k;m++,j++)
		    printf("   %d",buffer[j]);
		printf("\n");
		n--;
	    }
	    sleep(3);
	}
        x = x + incr; /* move pick box across screen */
    } /* end while */

    if( pause_flag ){
	printf("enter a char ");
	fscanf( stdin, "%c",&i);
    }
    Delete_str();
    Pexit();
    exit(0);
}

Set_view()
{
    int i, j;
    switch( ViewType ){
	case 0:
	    Ppersp_view( 0, fov, aspect, near, far);
	    break;
	case 1:
	    Portho_view( 0, left, right, bottom, top, near, far);
	    break;
	case 2:
	    Portho_2d_view( 0, left, right, bottom, top );
	    break;
	case 3:
	    Pwindow_view( 0, left, right, bottom, top, near, far );
	    break;
	case 4:
	    i = right - left;
	    j = top - bottom;
	    printf("viewport  0.. %d, 0.. %d\n",i,j);
	    Pviewport( 0, (int)(i), 0, (int)(j));
	    Pwindow_view( 0, left, right, bottom, top, near, far );
	    break;
    }
}

init_data()
{
    verts[0].x = -1.5; verts[0].y =3.0;   verts[0].z = -5.0;
    verts[1].x = 1.5;  verts[1].y =3.0;   verts[1].z = -5.0;
    verts[2].x = -1.5; verts[2].y = -.8;  verts[2].z = -5.0;
    verts[3].x = 1.5;  verts[3].y =0.0;   verts[3].z = -5.0;
    verts[4].x = -1.0; verts[4].y = -2.5; verts[4].z = -5.0;
    verts[5].x = 1.0;  verts[5].y = -3.0; verts[5].z = -5.0;
    verts[6].x = -4.0; verts[6].y =1.0;   verts[6].z = -5.0;
    verts[7].x = 5.5;  verts[7].y =1.0;   verts[7].z = -5.0;
    verts[8].x = -3.5; verts[8].y =0.0;   verts[8].z = -5.0;
    verts[9].x = 4.0;  verts[9].y =0.0;   verts[9].z = -5.0;
    verts[10].x = -5.0; verts[10].y = -1.0; verts[10].z = -5.0;
    verts[11].x = 4.0;  verts[11].y = -1.0; verts[11].z = -5.0;
}

Create_Poly( id_flag)
int id_flag; /* if true, then use as structure id */
{
    Fcoord3D	 vt[4];

    vt[0].x = -0.5; vt[0].y = 0.0; vt[0].z = -1.0;
    vt[1].x = -0.5; vt[1].y = -.9; vt[1].z = -1.0;
    vt[2].x = -0.2; vt[2].y = -.9; vt[2].z = -1.0;
    vt[3].x = -0.2; vt[3].y = 0.0; vt[3].z = -1.0;

    if( id_flag)
      Pcreate_struct(id_flag);
	Ppick_id(310);
	  Ppoly( 4, vt);
    if( id_flag)
      Pclose_struct();
}

Create_Text( id_flag)
int id_flag; /* if true, then use as structure id */
{
    int i;
    char str[10];

    if( id_flag)
	Pcreate_struct(id_flag);

    for(i=0; i<vct; i++){
	    Ppick_id(i+1); /* set pick id to vtx number, (base 1) */
	    sprintf(str,"%2d",(i+1));
	    Ptext(verts[i].x, verts[i].y, verts[i].z, str);
    }

    if( id_flag)
	Pclose_struct();
}

Create_Marker( id_flag)
int id_flag; /* if true, then use as structure id */
{
    int i;

    if( id_flag)
	Pcreate_struct(id_flag);

    for(i=0; i<vct; i++){
	    Ppick_id(201+i); /* set pick id to 200 + vtx number, (base 1) */
	    Pmarker( 1, &verts[i].x);
    }

    if( id_flag)
	Pclose_struct();
}

Delete_str()
{
    if(Pdelete_struct(1))
	printf("ERROR... deleting structure 1\n");
    if(Pdelete_struct(2))
	printf("ERROR... deleting structure 2\n");
    if(Pdelete_struct(3))
	printf("ERROR... deleting structure 3\n");
}

/*
 *                      +---------------------------+
 * ---------------------+         parse_line        +------------------------
 *                      +---------------------------+
 *  FUNCTION:
 *      Parses the command line.
 *
 *  USAGE:
 *      parse_line(argc, argv)
 *	int	   argc 	   (in)  - number of arg's
 *	char   	   argv 	   (in)  - pointer to arg list
 *
 *  DESCRIPTION:
 *	This routine will parse the command line for the following options:
 *	   opt  argument	type	  comment
 *	   ---  ------------    ----      ----------------------------------
 *	    -i incr		int	  increment (in pixels) to move pick box
 *	    -m view		int	  mode: 0=persp,1=ortho,2=ortho2,
 *					  3=window
 *	    -p 			void	  sets pause flag
 *	    -w width		float	  Sets window size
 *	    -z trans		float	  z translation value
 *	    -h near		float	  near (hither) plane
 *	    -y far		float	  far (yon) plane
 *	    -a aspect		float	  aspect ratio
 *	    -l left		float	  left
 *	    -r right		float	  right
 *	    -t top		float	  top
 *	    -b bottom		float	  bottom
 *                                
*/
int parse_line(argc,argv)
int argc;
char **argv;
{
     char *temp,ffile[32];
     extern char *optarg;
     extern int optind;
     int c;

    while((c=getopt(argc,argv,"a:i:h:y:l:r:b:t:m:t:pw:")) != EOF)
    {
	switch(c){
	    case 'a':
		  sscanf(optarg,"%f",&aspect);
		break;
	    case 'i':
		  sscanf(optarg,"%d",&incr);
		break;
	    case 'm':
		  sscanf(optarg,"%d",&ViewType);
		break;
	    case 'p':
		  pause_flag = 1;
		break;
	    case 'w':
		  sscanf(optarg,"%f",&win_width);
		  if( win_width > 0 && win_width < 1024)
			win_height = win_width;
		  else 
			win_width = win_height;
		break;
	    case 'h':
		  sscanf(optarg,"%f",&near);
		break;
	    case 'y':
		  sscanf(optarg,"%f",&far);
		break;
	    case 'l':
		  sscanf(optarg,"%f",&left);
		break;
	    case 'r':
		  sscanf(optarg,"%f",&right);
		break;
	    case 'b':
		  sscanf(optarg,"%f",&bottom);
		break;
	    case 't':
		  sscanf(optarg,"%f",&top);
		break;
	}
    }

}

Usage()
{
    printf("Error...  Usage is:\n");
    printf(" pck_test [options]\n");
    printf(" options: \n");
    printf("\t -m view \t(view type,0=persp,1=ortho)\n");
    printf("\t\t\t(2=ortho2,3=window)\n");
    printf("\t -i incr\t(number of pixels to move pick box by)\n");
    printf("\t -a aspect\t(aspect ratio, float)\n");
    printf("\t -l left\t(left coord, float)\n");
    printf("\t -r right\t(right coord, float)\n");
    printf("\t -t top\t\t(top coord, float)\n");
    printf("\t -b bottom\t(bottom coord, float)\n");
    printf("\t -h hither\t(set hither plane, float)\n");
    printf("\t -y yon\t\t(set yon plane, float)\n");
    printf("\t -w size\t(window size, int)\n");
    printf("\t -p\t\t(pause flag)\n");
}
