/* template:


    pexdpyinfo : PEX Info program
 
 	SHOgraphics
 	321 Soquel Way
 	Sunnyvale, CA  94086
 	
 	Copyright 1992, 1993.    All rights reserved.
 
    written by: Bob Schulte		date: January 1992

*******************************************************************************

	Program:   pexdpyinfo.c

	Synopsis:  This program will display various PEX default data.
		   The program is written to use the MIT PEXlib. 

	To compile:
		cc -o pexdpyinfo pexdpyinfo.c -lPEX -lX11

	Options:
		-e	Displays extended support (MPEX)
		-v	Displays available visuals, vendor name and 
			server version, available extensions.
		-r	Displays renderer dynamics.
		-l	Displays Lookup Table Info.
		-s	Displays the string mnemonic for Enumerated Info.
		-f	Displays available fonts.
		-m	Displays MatchRenderingTargets Information.
		-M	Displays various viewing matrices. Only for
			verifying view mapping matrices. Undocumented.
		-A all of the above

*******************************************************************************

    Permission to use, copy, modify, and distribute this software and
 its documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in
 supporting documentation, and that the name of ShoGraphics not be
 used in advertising or publicity pertaining to distribution of the 
 software without specific, written prior permission.

 SHOGRAPHICS DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 INCLUDING ALL EXPRESSED AND IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS. IN NO EVENT SHALL SHOGRAPHICS BE LIABLE FOR ANY DAMAGES OF ANY
 KIND, INCLUDING BUT NOT LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL 
 DAMAGES RESULTING FROM LOSS OF USE, DATA OR PROSPECTIVE PROFITS, WHETHER 
 IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, ARISING OUT 
 OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

*/

#include <stdio.h>
#include <math.h>
#include <PEXlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/Xlib.h>

#include <PEX.h>
#include <PEXproto.h>
#include <PEXprotost.h>
#include <PEXlib.h>

/***********GLOBALS****************************************************/
    Display 		 *displayptr;
    Window         root;
    int                  win_screen;
    int                  win_depth;
    int                  mrt_flag, verbose_flag, matrix_flag, fonts_flag, str_flag;
    int			 ext_flag;
    int			 lut_flag, rdr_flag;
#define ERROR_STR_LEN 2000
    char p_error_str[ERROR_STR_LEN];
    char		 *conn_id;

/****************************************************/
void usage()
	{
	printf(" USAGE: pexdpyinfo [ -l -v -m -f -s -r -A ]\n");
	printf("        (There MUST be at least 1 flag)\n");
    printf("            l:  print lookup tables.\n"); 
    printf("            v:  verbose\n"); 
    printf("            m:  print Matching Rendering Targets.\n"); 
    printf("            f:  print font info.\n"); 
    printf("            s:  displays string mnemonic for Enum Info\n"); 
    printf("            r:  Create renderer, get dynamics.\n"); 
    printf("            A:  ALL of the above.\n"); 
    printf("\n");
	exit(1);
	}

void mrt();
/*********************************************************************/
main (argc, argv)
int argc;
char **argv;
{
    int i;

    if (argc<2) usage();
    mrt_flag = verbose_flag = fonts_flag = matrix_flag =0;
    str_flag = ext_flag = 0;
    rdr_flag = lut_flag = 0;
    conn_id = NULL;

    for(i=1;i<argc;i++){
	if( strcmp(argv[i],"-e") == 0 )
	    ext_flag = 1;
	else if( strcmp(argv[i],"-l") == 0 )
	    lut_flag = 1;
	else if( strcmp(argv[i],"-v") == 0 )
	    verbose_flag = 1;
	else if( strcmp(argv[i],"-M") == 0 )
	    matrix_flag = 1;
	else if( strncmp(argv[i],"-m",2) == 0 )
	    mrt_flag = 1;
	else if( strcmp(argv[i],"-f") == 0 )
	    fonts_flag = 1;
	else if( strcmp(argv[i],"-s") == 0 )
	    str_flag = 1;
	else if( strcmp(argv[i],"-r") == 0 )
	    rdr_flag = 1;
	else if( strcmp(argv[i],"-A") == 0 )  /* ALL options */
	    mrt_flag = ext_flag = lut_flag = verbose_flag = 
		fonts_flag = str_flag = rdr_flag = 1;
	else if( strcmp(argv[i],"-display") == 0 ) {
	    i++;
	    conn_id = argv[i];
	}
	else usage();
    }

    Open_X_Window();
    root = RootWindow (displayptr, DefaultScreen (displayptr));
    init_pex_data(argc);

	if(mrt_flag) mrt();
    if( fonts_flag )
	Get_Fonts();

    if( matrix_flag )
	Get_Matrix_data();

    XCloseDisplay(displayptr);
    exit(0);
} 

/*
 *  -------------------------------- INIT_PEX_DATA --------------------------
 */

static PEXExtensionInfo *extension_info;
/************************************************************************/
init_pex_data(argc)
int argc;
{
  int stat,i,j,n,m_op,f_er,f_ev;
  short names[20];
  PEXImpDepConstant *valuesReturn;
  int   enumType;
  int   count;
  PEXExtensionInfo *inf;
  pexRenderer rdr;
	Window rr;
  char **xname;

    stat = PEXInitialize(displayptr, &extension_info, 
	ERROR_STR_LEN, p_error_str);
    if (stat) 
	{printf(" stst, error: %d %s\n",stat,p_error_str);exit(1);}
    else
	printf(" PEXInitialize returned False\n");
    inf = PEXGetExtensionInfo(displayptr);
    if (inf==NULL) 
        {printf("PEXGetExtensionInfo returned null\n");exit(1);}	
    
    printf("  major, minor version: %d %d\n",
	inf->major_version,inf->minor_version);
    printf("  release             : %d\n",inf->release);
    printf("  subset_info         : %x\n",inf->subset_info);
    printf("  vendor name         : %s\n",inf->vendor_name);
    printf("  major opcode        : %d\n",inf->major_opcode);
    printf("  first event         : %d\n",inf->first_event);
    printf("  first error         : %d\n",inf->first_error);
    printf("\n");

    if( verbose_flag ){
	xname = XListExtensions(displayptr, &n);
	printf(" Available Extensions: %d\n", n);
	for (i = 0; i < n; i++) {
            printf(" Extension %d = %s", i, xname[i]);
            j = XQueryExtension(displayptr, xname[i], &m_op, &f_ev, &f_er);
            printf("   Opcode = %d   event = %d   error = %d\n",
               m_op, f_ev, f_er);
	}
    }

    if( ext_flag || rdr_flag ) rdr = Get_Renderer_Dynamics(win_screen);

    names[0] = PEXIDDitheringSupported;
    names[1] = PEXIDTransparencySupported;
    names[2] = PEXIDMaxNonAmbientLights;
    names[3] = PEXIDMaxMarkerSize;
    names[4] = PEXIDMaxLineWidth;
    names[5] = PEXIDMinLineWidth;
    names[6] = PEXIDNominalLineWidth;
    names[7] = PEXIDNumSupportedLineWidths;
    names[8] = PEXIDMaxEdgeWidth;
    names[9] = PEXIDMinEdgeWidth;
    names[10] = PEXIDNominalEdgeWidth;
    names[11] = PEXIDNumSupportedEdgeWidths;
    names[12] = PEXIDMinMarkerSize;
    names[13] = PEXIDNominalMarkerSize;
    names[14] = PEXIDNumSupportedMarkerSizes;
    names[15] = PEXIDMaxNameSetNames;
    names[16] = PEXIDMaxNURBOrder;
    names[17] = PEXIDMaxModelClipPlanes;
    names[18] = PEXIDBestColorApprox;
    names[19] = PEXIDMaxHitsEventSupported;
    rr = RootWindow(displayptr,win_screen);
	stat=
    PEXGetImpDepConstants( displayptr, RootWindow(displayptr,win_screen),
		20, names, &valuesReturn);
	if (!stat)
	  {printf("GetImpDepConstants failed. returned %d\n",stat);return;}
    printf(" Dithering supported\t = %s\n",
	(valuesReturn[0].integer ? "True" : "False"));
    printf(" Transparency supported\t = %s\n",
		(valuesReturn[1].integer ? "True" : "False"));
    printf(" MaxLineWidth\t\t = %d, Min %d, Nominal %d, num %d\n",
	valuesReturn[4].integer, valuesReturn[5].integer,
	valuesReturn[6].integer,valuesReturn[7].integer);
    printf(" MaxEdgeWidth\t\t = %d, Min %d, Nominal %d, num %d\n",
	valuesReturn[8].integer, valuesReturn[9].integer,
	valuesReturn[10].integer,valuesReturn[10].integer);
    printf(" MaxMarkerSize\t\t = %d, Min %d, Nominal %d, num %d\n",
	valuesReturn[3].integer, valuesReturn[12].integer,
	valuesReturn[13].integer,valuesReturn[14].integer);
    printf(" MaxNonAmbientLights\t = %d\n", valuesReturn[2].integer);
    printf(" MaxNameSetNames\t = %d\n", valuesReturn[15].integer);
    printf(" MaxNURBOrder\t\t = %d\n", valuesReturn[16].integer);
    printf(" MaxModelClipPlanes\t = %d\n", valuesReturn[17].integer);
    printf(" BestColorApproximation\t = %d\n", valuesReturn[18].integer);
    printf(" MaxHitsEventSupported\t = %d\n", valuesReturn[18].integer);

    XFree( valuesReturn);

    printf("\n       PEXGetEnumTypeInfo\n");
    printf("      Value   Description            String Mnemonic\n");
    Get_Info(PEXETFloatFormat,"FloatFormat", 0);

    Get_Info(PEXETHLHSRMode,"HlhsrModes", 0);
    Get_Info(PEXETInteriorStyle,"InteriorStyle", 0);
    Get_Info(PEXETReflectionModel,"ReflectionModel", 0);
    Get_Info(PEXETSurfaceInterpMethod,"SurfaceInterpMethod", 0);
    Get_Info(PEXETPolylineInterpMethod,"PolylineInterpMethod", 0);
    Get_Info(PEXETMarkerType,"MarkerType", 0);
    Get_Info(PEXETLightType,"LightType", 0);
    Get_Info(PEXETColorApproxType,"ColorApproxType", 0);
    Get_Info(PEXETColorType,"ColorType", 0);
    Get_Info(PEXETColorApproxModel,"ColorApproxModel", 0);
    Get_Info(PEXETPickDeviceType,"PickDeviceType", 0);
    Get_Info(PEXETLineType,"LineType", 0);
    Get_Info(PEXETSurfaceEdgeType,"SurfaceEdgeType", 0);
    Get_Info(PEXETCurveApproxMethod,"CurveApproxMethod", 0);
    Get_Info(PEXETTrimCurveApproxMethod,"TrimCurveApproxMethod", 0);
    Get_Info(PEXETSurfaceApproxMethod,"SurfaceApproxMethod", 0);
    Get_Info(PEXETModelClipOperator,"ModelClipOperator", 0);
    Get_Info(PEXETRenderingColorModel,"RenderingColorModel", 0);

    if(!( inf->major_version == 5 && inf->minor_version == 0 ))
	Get_Info(PEXETEscape,"Escape", 1);

    Get_Info(PEXETGDP2D,"PEXGDP2D", 1);
    Get_Info(PEXETGDP,"PEXGDP", 1);
    Get_Info(PEXETGSE,"PEXGSE", 1);

    /* Get some PHIGS Wks Info */
    /* Check for Immed Mode Only, or an MPEX only version of server */
    if( inf->subset_info & 0x01 || (!(inf->subset_info & 0x80000000 ))){
	Get_Info(PEXETPromptEchoType,"PromptEchoType", 0);
	Get_Info(PEXETDisplayUpdateMode,"DisplayUpdateMode", 0);
    }

    if( inf->minor_version == 1 ){
	Get_Info(PEXETPickOneMethod,"PickOneMethod", 0);
	Get_Info(PEXETPickAllMethod,"PickAllMethod", 0);
    }

    if( lut_flag ) {
	    Get_Table_Info(PEXLUTColorApprox);
	    Get_Table_Info(PEXLUTColor);
	    Get_Table_Info(PEXLUTDepthCue);
	    Get_Table_Info(PEXLUTLight);
	    Get_Table_Info(PEXLUTView);
    }
    if( verbose_flag )
		XFreeExtensionList(xname);
}

/************************************************************************/
Get_Info(id, str, p_type)
int id;
char *str;
int p_type; /* Print type: 0=decimal, 1=hexidecimal */
{
  int   stat, i, enum_type;
  unsigned long *count;
  unsigned long val;
  int idd=id;
  PEXEnumTypeDesc *info,*ptr;

    enum_type = PEXETIndex;
    if( str_flag ) enum_type = PEXETAll;
  
    stat= PEXGetEnumTypeInfo( displayptr, RootWindow(displayptr,win_screen),
	1, &idd, enum_type, &count, &info);
	if (!stat) 
	    {printf("GetEnumTypeInfo failed. returned %d\n",stat);return;}
    ptr = info;
    if( *count == 0 ){
	printf(" No support:  %s\n", str);
	return;
    }
    if( str_flag ){
	for(i=0;i < *count;i++,ptr++){
	    val = ptr->index;
	    if( p_type == 1 ){
	        if( val & 0x8000) val = -val;
	        printf("%9x     %s  %s\n", val,str,ptr->descriptor);
	    } else
		printf("%9d     %s  %s\n", val,str,ptr->descriptor);
	    /* printf("%s %d  %s\n",str,ptr->index,ptr->descriptor); */
	}
    } else {
	for(i=0;i < *count;i++,ptr++){
	    val = ptr->index;
	    if( p_type == 1 ){
	        if( val & 0x8000) val = -val;
	        val = (unsigned long) (ptr->index & 0xffff);
		printf("%9x     %s\n", val,str);
	    } else
		printf("%9d     %s\n", val,str);
	}
    }

    PEXFreeEnumInfo(1,count,info);
}

/************************************************************************/
Open_X_Window()
{
    if ((displayptr = XOpenDisplay (conn_id)) == NULL)
    {
        printf("error opening display\n");exit(1);
    }

    win_screen   = DefaultScreen (displayptr);
    win_depth    = DefaultDepth (displayptr, win_screen);
    /*if( verbose_flag ) Get_Visuals(win_screen);*/
}

/************************************************************************/
Get_Visuals(screen_num)
int screen_num;
{
  XVisualInfo vTemplate, *visualList;
  int i, n_visuals, class;
  Visual *win_vis;

    win_vis = DefaultVisual( displayptr, screen_num);
    printf("Default Visual class ");
#if defined(__cplusplus) || defined(c_plusplus)
    Print_class(win_vis->c_class);
#else
    Print_class(win_vis->class);
#endif
    printf(", depth %d, entries %d\n",
	win_vis->bits_per_rgb,win_vis->map_entries);
    /* Print_Visuals( win_vis, 1); */

    vTemplate.screen = screen_num;
    /* vTemplate.depth = 8; */

    visualList = XGetVisualInfo( displayptr, VisualScreenMask,
	&vTemplate, &n_visuals);

    if( n_visuals == 0 )
	printf("\nERROR...matching visuals\n");

    Print_Visuals( visualList, n_visuals);
}

/************************************************************************/
Print_Visuals( visualList, n_visuals)
XVisualInfo *visualList;
int	n_visuals;
{
    int i, c_class;

    printf(" visual ");
    /* printf("screen "); */
    printf(" depth     class");
    printf("     r_mask         g_mask         b_mask       size\n");
    for(i=0;i<n_visuals; i++){
	printf(" %5d",i);
	/* printf(" %5d",visualList[i].screen); */
	printf(" %5d ",visualList[i].depth);

#if defined(__cplusplus) || defined(c_plusplus)
	c_class = visualList[i].c_class;
#else
	c_class = visualList[i].class;
#endif
	Print_class( c_class);
	printf("  ");
	d2bin(visualList[i].red_mask,8);   printf("  ");
	d2bin(visualList[i].green_mask,8); printf("  ");
	d2bin(visualList[i].blue_mask,8);
	printf(" %4d",visualList[i].colormap_size);
	printf("\n");
    }
    printf("\n");
}

/************************************************************************/
Print_class( c_class)
int c_class;
{
	if( c_class == DirectColor) printf("  DirectColor");
	else
	if( c_class == PseudoColor) printf("  PseudoColor");
	else
	if( c_class ==  TrueColor)  printf("    TrueColor");
	else
	if( c_class ==  GrayScale)  printf("    GrayScale");
	else
	if( c_class ==  StaticGray) printf("   StaticGray");
	else
	if( c_class == StaticColor) printf("  StaticColor");
	else
				    printf(" Unknown Class");

	return;
}

/************************************************************************/
void mrt()
	{
	int stat,count,i;
	PEXRenderingTarget *targs,*sav_targs;
	char tstr[500];
	stat=
	PEXMatchRenderingTargets( displayptr, root, 0, PEXAnyDrawable,
	    NULL,50,&count,&targs);
	if (!stat) {printf("MatchRenderingTargets failed\n");return;}
	if (count==0) 
	    {printf(" MatchRenderingTargets failed (count=0)\n");
	    return;}
	printf("Matched %d Rendering Targets\n",count);
	sav_targs = targs;
	for (i=0; i<count; i++,targs++)
	    {
	    switch(targs->type)
		{
		case PEXWindowDrawable: 
			strcpy(tstr,"WindowDrawable");
			break;
		case PEXPixmapDrawable: 
			strcpy(tstr,"PixmapDrawable");
			break;
		case PEXBufferDrawable: 
			strcpy(tstr,"BufferDrawable");
			break;
		default:                
			strcpy(tstr,"UNKNOWN_Drawable");
			break;
		}
		printf("depth = %d, type = %s, visual = %x, class %d\n",
		targs->depth,tstr,targs->visual,targs->visual->class);
	    }
	XFree(sav_targs);
	}

/************************************************************************/
Get_Fonts()
{
  int i, n;
  char **arr;

    arr = PEXListFonts( displayptr, "*", 20, &n);

    if( arr==NULL || n==0){
	printf("ERROR...PEXListFonts error\n");
	return;
    }
  for (i=0; i<n; i++)
  	{
	printf("%s\n",arr[i]);
  }
  PEXFreeFontNames(n, arr);
}

/************************************************************************/
long
Get_Renderer_Dynamics()
{
    unsigned long tRet, nRet, aRet;
    pexRenderer rdr;
    PEXRendererAttributes pexAttr;

    rdr = PEXCreateRenderer( displayptr, root, 0, &pexAttr);

    if(! PEXGetRendererDynamics( displayptr, rdr, &tRet, &nRet, &aRet)){
	printf("ERROR... GetRendererDynamics\n");
	return rdr;
    }

    if( tRet == 0 )
	printf(" All Lookup Table items  are dynamic\n");
    else {
	if( tRet & PEXRDTMarkerBundle )
		printf(" PEXMarkerBundleLUT\t not dynamic\n");
	else
		printf(" PEXMarkerBundleLUT\t are dynamic\n");
	if( tRet & PEXRDTTextBundle )   
		printf(" PEXTextBundleLUT\t not dynamic\n");
	else
		printf(" PEXTextBundleLUT\t are dynamic\n");
	if( tRet & PEXRDTLineBundle )   
		printf(" PEXLineBundleLUT\t not dynamic\n");
	else
		printf(" PEXLineBundleLUT\t are dynamic\n");
	if( tRet & PEXRDTInteriorBundle ) 
		printf(" PEXInteriorBundleLUT\t not dynamic\n");
	else
		printf(" PEXInteriorBundleLUT\t are dynamic\n");
	if( tRet & PEXRDTEdgeBundle )   
		printf(" PEXEdgeBundleLUT\t not dynamic\n");
	else
		printf(" PEXEdgeBundleLUT\t are dynamic\n");
	if( tRet & PEXRDTViewTable )
		printf(" ViewTable\t\t not dynamic\n");
	else
		printf(" ViewTable\t\t are dynamic\n");
	if( tRet & PEXRDTColorTable)
		printf(" ColorTable\t\t not dynamic\n");
	else
		printf(" ColorTable\t\t are dynamic\n");
	if( tRet & PEXRDTDepthCueTable)
		printf(" DepthCueTable\t\t not dynamic\n");
	else
		printf(" DepthCueTable\t\t are dynamic\n");
	if( tRet & PEXRDTLightTable )
		printf(" LightTable\t\t not dynamic\n");
	else
		printf(" LightTable\t\t are dynamic\n");
	if( tRet & PEXRDTColorApproxTable )
		printf(" ColorApproxTable\t not dynamic\n");
	else
		printf(" ColorApproxTable\t are dynamic\n");
	if( tRet & PEXRDTPatternTable )
		printf(" PatternTable\t\t not dynamic\n");
	else
		printf(" PatternTable\t\t are dynamic\n");
	if( tRet & PEXRDTTextFontTable )
		printf(" TextFontTable\t\t not dynamic\n");
	else
		printf(" TextFontTable\t\t are dynamic\n");
    }

    if( nRet == 0 )
	printf(" All NameSet items       are dynamic\n");
    else {
	if( nRet & PEXRDNHighlightNameSet ) 
		printf(" HighlightNameset\t not dynamic\n");
	else
		printf(" HighlightNameset\t are dynamic\n");
	if( nRet & PEXRDNInvisibilityNameSet ) 
		printf(" InvisibilityNameset\t not dynamic\n");
	else
		printf(" InvisibilityNameset\t are dynamic\n");
	if( nRet & PEXRDNPickNameSet ) 
		printf(" PickNameset\t\t not dynamic\n");
	else
		printf(" PickNameset\t\t are dynamic\n");
	if( nRet & PEXRDNPickNameSetContents ) 
		printf(" PickNamesetContents\t not dynamic\n");
	else
		printf(" PickNamesetContents\t are dynamic\n");
    }
    if( aRet == 0 )
	printf(" All Attribute items     are dynamic\n");
    else {
	if( aRet & PEXRDAHLHSRMode )
		printf(" PEXRDTHlhsrMode\t not dynamic\n");
	else
		printf(" PEXRDTHlhsrMode\t are dynamic\n");
	if( aRet & PEXRDANPCSubVolume )
		printf(" NpcSubvolume\t\t not dynamic\n");
	else
		printf(" NpcSubvolume\t\t are dynamic\n");
	if( aRet & PEXRDAViewport )
		printf(" Viewport\t\t not dynamic\n");
	else
		printf(" Viewport\t\t are dynamic\n");
	if( aRet & PEXRDAClipList )
		printf(" ClipList\t\t not dynamic\n");
	else
		printf(" ClipList\t\t are dynamic\n");
	if( aRet & PEXRDAEchoMode )
		printf(" EchoMode\t\t not dynamic\n");
	else
		printf(" EchoMode\t\t are dynamic\n");
    }
	   
    printf(" Masks : table %x",tRet);
    printf(", nameset %x",nRet);
    printf(", attrib %x\n",aRet);

    Get_Renderer_Attributes(rdr);
    return rdr;
}

/************************************************************************/
int
Get_Renderer_Attributes(rdr)
pexRenderer rdr;
{
    int i;
    unsigned long aMask;
    PEXRendererAttributes *pa;

	aMask=
	    PEXRAPipelineContext | PEXRACurrentPath |   PEXRAMarkerBundle |
	    PEXRATextBundle      | PEXRALineBundle |    PEXRAInteriorBundle |
	    PEXRAEdgeBundle      | PEXRAViewTable |     PEXRAColorTable |
	    PEXRADepthCueTable   | PEXRALightTable |    PEXRAColorApproxTable |
	    PEXRAPatternTable    | PEXRATextFontTable | PEXRAHighlightIncl |
	    PEXRAHighlightExcl  |PEXRAInvisibilityIncl| PEXRAInvisibilityExcl|
	    PEXRARendererState |   PEXRAHLHSRMode |     PEXRANPCSubVolume |
	    PEXRAViewport | 	   PEXRAClipList | 	PEXRAPickIncl |
	    PEXRAPickExcl |        PEXRAPickStartPath | PEXRABackgroundColor |
	    PEXRAClearImage |      PEXRAClearZ | 	PEXRAEchoMode;
	
    pa = PEXGetRendererAttributes( displayptr, rdr, aMask);
  	if (pa==NULL)
  		{
		printf("ERROR... GetRendererAttributes\n");
		return 0;
   		}
	if( aMask & PEXRAPipelineContext )
		printf(" PEXRAPipelineContext\t %x\n",pa->pipeline_context);
	if( aMask & PEXRAMarkerBundle )
		printf(" PEXMarkerBundleLUT\t %x\n",pa->marker_bundle);
	if( aMask & PEXRATextBundle )   
		printf(" PEXTextBundleLUT\t %x\n",pa->text_bundle);
	if( aMask & PEXRALineBundle )   
		printf(" PEXLineBundleLUT\t %x\n",pa->line_bundle);
	if( aMask & PEXRAInteriorBundle ) 
		printf(" PEXInteriorBundleLUT\t %x\n",pa->interior_bundle);
	if( aMask & PEXRAEdgeBundle )   
		printf(" PEXEdgeBundleLUT\t %x\n",pa->edge_bundle);

	if( aMask & PEXRAViewTable )
		printf(" ViewTable\t\t %x\n",pa->view_table);
	if( aMask & PEXRAColorTable)
		printf(" ColorTable\t\t %x\n",pa->color_table);
	if( aMask & PEXRADepthCueTable )
		printf(" DepthCueTable\t\t %x\n", pa->depth_cue_table);
	if( aMask & PEXRALightTable ) 
		printf(" LightTable\t\t %x\n",pa->light_table);
	if( aMask & PEXRAColorApproxTable )
		printf(" ColorApproxTable\t %x\n",pa->color_approx_table);
	if( aMask & PEXRAPatternTable )
		printf(" PatternTable\t\t %x\n",pa->pattern_table);
	if( aMask & PEXRATextFontTable )
		printf(" TextFontTable\t\t %x\n",pa->text_font_table);

	if( aMask & PEXRAHighlightIncl )
		printf(" HilightIncl\t\t %x\n",pa->highlight_incl);
	if( aMask & PEXRAHighlightExcl )
		printf(" HiliteExcl\t\t %x\n",pa->highlight_excl);
	if( aMask & PEXRAInvisibilityIncl )
		printf(" InvisibilityIncl\t %x\n",pa->invisibility_incl);
	if( aMask & PEXRAInvisibilityExcl )
		printf(" InvisibilityExcl\t %x\n",pa->invisibility_excl);

	if( aMask & PEXRAHLHSRMode )
		printf(" PEXDynHlhsrMode:\t");
	switch(pa->hlhsr_mode)
		{
		case PEXHLHSROff:       printf(" PEXHLHSROff\n");break;
		case PEXHLHSRZBuffer:   printf(" PEXHLHSRZbuffer\n");break;
		case PEXHLHSRZBufferID: printf(" PEXHLHSRZbufferID\n");break;
		case PEXHLHSRPainters:  printf(" PEXHLHSRPainters\n");break;
		case PEXHLHSRScanline:  printf(" PEXHLHSRScanline\n");break;
		case PEXHLHSRHiddenLineOnly: 
				printf(" PEXHLHSRHiddenLineOnly\n");break;
		}
	if( aMask & PEXRANPCSubVolume )
		printf(" NpcSubvolume\t %x\n",pa->npc_subvolume);
	if( aMask & PEXRAViewport ) {
	    printf(" Viewport\t\t min x %d, y %d; max x %d, y %d, useDraw %d\n",
	        pa->viewport.min.x,
		pa->viewport.min.y, pa->viewport.max.x,
	        pa->viewport.max.y, pa->viewport.use_drawable);
	}
	if( aMask & PEXRAClipList ) {
		printf(" ClipList\t\t %d,%d\n", pa->clip_list.count);
		for(i=0;i<pa->clip_list.count; i++)
		    printf("\t\txmin %d, ymin %d, xmax %d, ymax %d\n",
			pa->clip_list.rectangles[i].xmin,
			pa->clip_list.rectangles[i].ymin,
			pa->clip_list.rectangles[i].xmax,
			pa->clip_list.rectangles[i].ymax);
	}
	   
  PEXFreeRendererAttributes(pa);
    return 1;
}

/*
 *	Routine:	d2bin
 *	Purpose:	converts decimal value to binary printout
 */
#define B_SIZE	33

/************************************************************************/
d2bin(val, size)
unsigned long val;
int size;		/* amount of bits concerned about */
{
    int i, orig_val, start, end, shift;
    char buffer[B_SIZE];

    orig_val = val;
    shift = 0;

    for(i=0; i< B_SIZE;i++)buffer[i]='0';

    for(i=B_SIZE-1; i>=0; i--){
	if( val & 1) buffer[i] = '1';
	val >>= 1;
    }

    for(start=0;start<B_SIZE && buffer[start] != '1'; )
	start++;

    if( start < B_SIZE ){
	for(end=start+1;end<B_SIZE && buffer[end] != '0'; )
	    end++;
	shift = B_SIZE - end;
    } else {
	start=B_SIZE-1-size;
	end=B_SIZE-1;
	shift = 0;
    }

    if( end - start > size || end - start == 0 ||
        start >= end  || shift < size ){
	    start=B_SIZE-1-size;
	    end=B_SIZE-1;
	    shift = 0;
    }
    /* make sure shift is multiple of size */
    if( (i=shift % size) != 0 ){
	if( (int) (shift/size) != 0){
	    shift = shift + i;
	    start = start - i;
	}
    }

    for( i=start;i<end;i++)
	printf("%c",buffer[i]);

    if( shift != 0 )
	printf(" >>%2d",shift);
    else
	printf("     ");
}

/************************************************************************/
Get_Table_Info( type )
int	type;
{
  int stat,i, count, num;
  pexTableInfo	ptr;
  char *pptr, *cptr;
  PEXColorApproxEntry *clr_ptr;
  PEXColorEntry *ce_ptr;

	stat=
   PEXGetTableInfo(displayptr,RootWindow(displayptr,win_screen), type,
	&ptr);
	if (!stat) {printf("GetTableInfo failed\n");exit(1);}

    switch( type ){
	case PEXLUTLineBundle:
	case PEXLUTMarkerBundle:
	case PEXLUTTextBundle:
	case PEXLUTInteriorBundle:
	case PEXLUTEdgeBundle:
	case PEXLUTPattern:
	case PEXLUTTextFont:
		break;
	case PEXLUTColor:
		printf("Color LUT\n");
		print_table_info( ptr);
   		PEXGetPredefinedEntries(displayptr,
			RootWindow(displayptr,win_screen), type,
			ptr.predefinedMin, ptr.numPredefined,
			&cptr);
		num = ptr.numPredefined;
		ce_ptr = (PEXColorEntry *)cptr;
		for(i=0;i<num;i++){
		   printf("     type %d, value %f %f %f\n",
			(int)ce_ptr->type, ce_ptr->value.rgb.red,
			ce_ptr->value.rgb.green, ce_ptr->value.rgb.blue);
		}
		break;
	case PEXLUTColorApprox:
		printf("Color Approx LUT\n");
		print_table_info( ptr);
   		PEXGetPredefinedEntries(displayptr,
			RootWindow(displayptr,win_screen), type,
			ptr.predefinedMin, ptr.numPredefined,
			&cptr);
		num = ptr.numPredefined;
		clr_ptr = (PEXColorApproxEntry *)cptr;
		for(i=0;i<num;i++){
		   printf("     type %d, model %d\n",
			(int)clr_ptr->type,
			(int)clr_ptr->model);
		   printf("\tmax1 %d, max2 %d max3 %d, dither %d\n",
			(short)clr_ptr->max1,
			(short)clr_ptr->max2,
			(short)clr_ptr->max3,
			(int)(clr_ptr)->dither);
		   printf("\tmult1 %d, mult2 %d, mult3 %d\n",
			(int)clr_ptr->mult1,
			(int)clr_ptr->mult2,
			(int)clr_ptr->mult3);
		   printf("\tmult1 %f, mult2 %f, mult3 %f\n",
			(float)clr_ptr->mult1,
			(float)clr_ptr->mult2,
			(float)clr_ptr->mult3);
		   printf("\twt1 %f, wt2 %f, wt3 %f, basePixel %d\n",
			(float)(clr_ptr)->weight1,
			(float)(clr_ptr)->weight2,
			(float)(clr_ptr)->weight3,
			(int)(clr_ptr)->base_pixel);
		    clr_ptr++;
		}
		break;
	case PEXLUTView:
			printf("View LUT\n");
			print_table_info( ptr);
   			PEXGetPredefinedEntries(displayptr,
				RootWindow(displayptr,win_screen), type,
				ptr.predefinedMin, ptr.numPredefined, 
				&cptr);
			num = ptr.numPredefined;
			pptr = cptr;
			for(i=0;i<num;i++){
			   printf("\tclipFlags %d, min x %f, y %f, z %f\n",
				((pexViewEntry *)pptr)->clipFlags,
				((pexViewEntry *)pptr)->clipLimits.minval.x,
				((pexViewEntry *)pptr)->clipLimits.minval.y,
				((pexViewEntry *)pptr)->clipLimits.minval.z);
			   printf("\tmax x %f, y %f, z %f\n",
				((pexViewEntry *)pptr)->clipLimits.maxval.x,
				((pexViewEntry *)pptr)->clipLimits.maxval.y,
				((pexViewEntry *)pptr)->clipLimits.maxval.z);
			    pptr += sizeof(pexViewEntry);
			}
		break;
	case PEXLUTLight:
		printf("\nLight LUT");
		print_table_info( ptr);
   		PEXGetPredefinedEntries(displayptr,
			RootWindow(displayptr,win_screen), type,
			ptr.predefinedMin, ptr.numPredefined, 
			&cptr);
		num = ptr.numPredefined;
		pptr = cptr;
		for(i=0;i<num;i++){
		   printf("\ttype %d is %s\n",
			((pexLightEntry *)pptr)->lightType,
			((pexLightEntry *)pptr)->lightType==
			PEXLightAmbient ?"Ambient" : "Non-Ambient");
		  if( ((pexLightEntry *)pptr)->lightType == 
			PEXLightWCPoint)
		   printf("\tpoint x %f, y %f, z %f\n",
			((pexLightEntry *)pptr)->point.x,
			((pexLightEntry *)pptr)->point.y,
			((pexLightEntry *)pptr)->point.z);
		   printf("\tcolor %8x\n",
			((pexLightEntry *)pptr)->lightColor.colorType);
		    pptr += sizeof(pexLightEntry);
		}
		break;
	case PEXLUTDepthCue:
		printf("DepthCue LUT\n");
		print_table_info( ptr);
   		PEXGetPredefinedEntries(displayptr,
			RootWindow(displayptr,win_screen), type,
			ptr.predefinedMin, ptr.numPredefined, 
			&cptr);
		num = ptr.numPredefined;
		pptr = cptr;
		for(i=0;i<num;i++){
		   printf("\tmode %d, frontPlane %f, backPlane %f\n",
			((pexDepthCueEntry *)pptr)->mode,
			((pexDepthCueEntry *)pptr)->frontPlane,
			((pexDepthCueEntry *)pptr)->backPlane);
		   printf("\tfrontScaling %f, backScaling %f\n",
			((pexDepthCueEntry *)pptr)->frontScaling,
			((pexDepthCueEntry *)pptr)->backScaling);

		    pptr += sizeof(pexDepthCueEntry);
			}
		break;
	default: printf("bad Get_Table_Info id: %n\n",type);break;
    }
    PEXFreeTableEntries( cptr);
}

/************************************************************************/
print_table_info(ptr)
pexTableInfo ptr;
{
	printf("definable %d, numPredefined %d\n",
		ptr.definableEntries,ptr.numPredefined);
	printf("predefined min %d, predefined max %d\n",
		ptr.predefinedMin,ptr.predefinedMax);
}

/************************************************************************/
Get_Matrix_data()
{
	pexMatrix pmat;
	double fovy, distance, aspect, near, far;
	pexCoord3D from, to, up;
	int i;

	fovy = 90.0*3.141592/180.0; distance = 1.0;
	near = -1.0; far = -1000.0; aspect = 1.0;
	/* distance = 90.0; */
	/* near =  0.5; far = -0.5; aspect = 1.0; */
	PEXPerspProjMatrix(fovy, distance, aspect, near, far, pmat);
	printf("Persp matrix\n");
	for(i=0;i<4;i++)
	    printf("%f %f %f %f\n",
            pmat[i][0], pmat[i][1], pmat[i][2], pmat[i][3]);
	near = -1.0; far = -1000.;
	PEXOrthoProjMatrix(1.0, aspect, near, far, pmat);
	printf("Ortho matrix\n");
	for(i=0;i<4;i++)
	    printf("%f %f %f %f\n",
            pmat[i][0], pmat[i][1], pmat[i][2], pmat[i][3]);
        from.x = from.y = to.x = to.y = to.z = up.x = up.z = 0.0;
        from.z = 90.0; up.y = 1.0;
	PEXLookAtViewMatrix(&from,&to,&up,pmat);
	printf("Lookat matrix\n");
	for(i=0;i<4;i++)
	    printf("%f %f %f %f\n",
            pmat[i][0], pmat[i][1], pmat[i][2], pmat[i][3]);
}

