/*
 ******************************************************************************
 ******************************************************************************
 *****                        read_off.c                                  *****
 *****									  *****
 *****		      File reading utility for OFF .geom files 		  *****
 *****									  *****
 *****									  *****
 *                                                                            *
 *	Copyright 1992, 1993. All Rights Reserved                             *
 *			SHOgraphics					      *
 *			321 Soquel Way           			      *
 *			Sunnyvale, CA     94086   			      *
 *                                                                            *
 *****									  *****

    Permission to use, copy, modify, and distribute this software and
 its documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in
 supporting documentation, and that the name of SHOgraphics not be
 used in advertising or publicity pertaining to distribution of the 
 software without specific, written prior permission.

 SHOGRAPHICS DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 INCLUDING ALL EXPRESSED AND IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS. IN NO EVENT SHALL SHOGRAPHICS BE LIABLE FOR ANY DAMAGES OF ANY
 KIND, INCLUDING BUT NOT LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL 
 DAMAGES RESULTING FROM LOSS OF USE, DATA OR PROSPECTIVE PROFITS, WHETHER 
 IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, ARISING OUT 
 OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

 ************************************************************************ */

#include <stdio.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <PEXtk.h>

#define USE_DB

#include <strings.h>
#include <math.h>
#include "obj_def.h"
#include "include/database.h"

int verbose_flag;

	static char *get_next_float();

/*
 *  ROUTINE:	read_off
 *  PURPOSE:	reads a .geom file
 */
read_off( argv, Obj_data)
char		*argv;
ObjData		*Obj_data;
{
    FILE *fp2, *fpcol, *fpvcol, *fopen();
    FILE *init_read_data();
    char buffer[512],objname2[128], *bufptr, *ptr;
    char *prim_name[128];
    char *objname;
    int value;
    int i, j, lcount, num_poly;
    float scale;
    double atof();
    int itmp;
    Fcoord4D    *verts;
    Fcoord3D    *vertt, *vertn;
    unsigned short  *vert_indices;
    unsigned short  *vt_indices;
    unsigned short  *vn_indices;
    unsigned short  *facet_Count, *norm_Count;
    int             *poly_cnt, *vct, *vtct, *vnct, *numEdges;
    int		    vcct, pcct;
    float		z_trans;
    Fcoord3D 	*ctr;
    Fcoord3D 	*global_pos;
    Fcoord3D 	*global_min;
    Fcoord3D 	*global_max;

	z_trans	   = Obj_data->z_trans;
	ctr	   = &Obj_data->ctr;
	global_pos = &Obj_data->global_pos;
	global_max = &Obj_data->global_max;
	global_min = &Obj_data->global_min;
	poly_cnt   = &Obj_data->poly_cnt;
	vct	   = &Obj_data->vct;
	vtct	   = &Obj_data->vtct;
	vnct	   = &Obj_data->vnct;
	numEdges   = &Obj_data->numEdges;
	verts	   = Obj_data->verts;
	vertt	   = Obj_data->vertt;
	vertn	   = Obj_data->vertn;
	vert_indices= Obj_data->vert_indices;
	vt_indices = Obj_data->vt_indices;
	vn_indices = Obj_data->vn_indices;
	facet_Count= Obj_data->facet_Count;
	norm_Count = Obj_data->norm_Count;
	fpcol = NULL;

    scale = 1.0;
    objname = argv;

	strcpy(objname2,objname);
						/* read in file.obj */
    if((fp2 = fopen(objname, "r")) == NULL)  {
	strcat(objname2,".geom");
	strcpy(prim_name,objname);
	
    	if((fp2 = fopen(objname2, "r")) == NULL)  {
	  printf("Error: can't open object %s\n", objname);
	  exit(-1);
	}
    } else {
	strncpy(prim_name,objname,strlen(objname)-5);
    }

	Obj_data->file_type = 1;
	read_aoff( prim_name, Obj_data);
	vcct = 0;
	vcct = read_vcol( prim_name, Obj_data);
	if( vcct > 0 )
	    Obj_data->mask |= DB_VCLR_MASK;


    lcount = 0;
    if(fgets(buffer, 128, fp2) == NULL)
	return 1;
      lcount++;

    sscanf(buffer,"%d\t%d\t%d",vct, &num_poly, &itmp);

	for(i=0;i<*vct;i++) {
	    if(fgets(buffer, 128, fp2) == NULL)
		break;
	    lcount++;

	bufptr = buffer;
	bufptr = get_next_float( bufptr, &verts[i].x);
	bufptr = get_next_float( bufptr, &verts[i].y);
	bufptr = get_next_float( bufptr, &verts[i].z);

		verts[i].x *= scale;
		verts[i].y *= scale;
		verts[i].z = verts[i].z + z_trans;
		/*
		 * Find the min's and max's
		 */
		if( verts[i].x > global_max->x)
			global_max->x = verts[i].x;
		if( verts[i].x < global_min->x)
			global_min->x = verts[i].x;
		if( verts[i].y > global_max->y)
			global_max->y = verts[i].y;
		if( verts[i].y < global_min->y)
			global_min->y = verts[i].y;
		if( verts[i].z > global_max->z)
			global_max->z = verts[i].z;
		if( verts[i].z < global_min->z)
			global_min->z = verts[i].z;
	}
	for(i=0;i<0;i++){
	    sscanf(buffer, "%f %f %f\n",
		    &vertn[i].x, &vertn[i].y, &vertn[i].z);
	}
	for(i=0;i<0;i++){
	    sscanf(buffer, "%f %f %f\n",
		    &vertt[i].x, &vertt[i].y, &vertt[i].z);
	}

	/* try to read the .pcol file */
	if( vcct == 0 )
	    fpcol = init_read_data( prim_name, Obj_data, ".pcol", &pcct);
	if( fpcol ){
	    if( pcct != num_poly )
	        fpcol = NULL;	/* error... not enough colors */
	    if((Obj_data->pcol = (rgbFloat *) 
		malloc( sizeof(int) * pcct)) == NULL )
		fpcol = NULL;
	    if( fpcol ) /* if everything still ok, set mask */
		Obj_data->mask |= DB_PCLR_MASK;
	}

	pcct = 0;	/* init current poly color count */

	for(i=0;i<num_poly;i++){
	    if(fgets(buffer, 128, fp2) == NULL)
	      break;

	    bufptr = strchr( buffer, '\n'); /* clear out the newline (\n) */
	    if( bufptr == NULL ) break;
	    *bufptr = 0;

	    sscanf(buffer,"%d",&j);	/* get number of indices */
	    facet_Count[i] = j;

	    bufptr = buffer;		/* look for first whitespace */
	    while( bufptr && !isspace(*bufptr)) bufptr++;
	    if( bufptr == NULL ) break;

	    (*poly_cnt)++;
	    for(; j>0; j--){
		/* look past spaces */
		while( bufptr &&  isspace(*bufptr) ) bufptr++;
		if( bufptr == NULL ) break;

		if( isdigit(*bufptr) )
		    value = atoi(bufptr);
		else {break;}

		if( value == 0 ){ break;}
		if( value > *vct+1 || value < 1 )
		    error_index("polygon","vertex",lcount);

		vert_indices[*numEdges] = value-1;
		(*numEdges)++;

		/* look for next non-digit */
	        while( bufptr && isdigit(*bufptr)) bufptr++;
		if( bufptr == NULL ) break;
	    }
		norm_Count[*poly_cnt]  = 0;
	    if( Obj_data->flip_flag ){	/* swap indices */
		/*
		 * To swap, start at "beginning", and traverse through
		 * half the list, swapping each index
		 */
		for(j = *numEdges-facet_Count[i], itmp= *numEdges-1; 
		    j<(*numEdges-facet_Count[i])+facet_Count[i]/2;j++,itmp--){
		    value = vert_indices[j];
		    vert_indices[j] = vert_indices[itmp];
		    vert_indices[itmp] = value;
	        }
	    }
	    if( fpcol ){
		fscanf( fpcol, "%f %f %f\n", 
		    &Obj_data->pcol[pcct].red, 
		    &Obj_data->pcol[pcct].green, &Obj_data->pcol[pcct].blue);
		    Obj_data->r = Obj_data->pcol[pcct].red;
		    Obj_data->g = Obj_data->pcol[pcct].green;
		    Obj_data->b = Obj_data->pcol[pcct].blue;
		pcct++;
	    }
#ifdef USE_DB
	    DB_add_element( argv, EL_TY_POLY, Obj_data);
#endif
    } /* end for num_poly */

    printf("%s contains %d vertices, %d vert norm's, %d polys, %d edges\n\n", 
		objname2, *vct, *vnct, *poly_cnt, *numEdges );

    global_pos->x = (float)fabs(global_max->x);
    if( fabs(global_min->x) > global_pos->x )
	global_pos->x = (float)fabs(global_min->x);

    global_pos->y = (float)fabs(global_max->y);
    if( fabs(global_min->y) > global_pos->y )
	global_pos->y = (float)fabs(global_min->y);

    global_pos->z = (float)fabs(global_max->z);
    if( fabs(global_min->z) > global_pos->z )
	global_pos->z = (float)fabs(global_min->z);

    if( verbose_flag ){
	printf(" global_x %f, global_y %f, global_z %f\n",
		global_pos->x,global_pos->y,global_pos->z);
	printf(" global_max->x %f, global_max->y %f, global_max->z %f\n",	
		global_max->x,global_max->y,global_max->z);
	printf(" global_min->x %f, global_y_min %f, global_z_min %f\n",
		global_min->x,global_min->y,global_min->z);
    }

    fclose(fp2);
    if( fpcol )
	fclose(fpcol);
    else if( vcct == 0 )
	read_ipcol( prim_name, Obj_data);

}

/*
 *  ROUTINE:	get_next_float
 *  PURPOSE:	Gets the next floating point value in string 's', and
 *		stores it in 'value'.
 *  RETURNS:	pointer to current position in buffer after getting
 *		the floating point value.
 */
static char *
get_next_float( s, value )
char *s;
float *value;
{


	while( s && isspace(*s)) s++;
	if( s == NULL ){
	    *value = 0.0;
	    return NULL;
	}

	*value = (float) atof(s);
	while( s && !isspace(*s) ) s++;
	return s;
}

int
read_vnorm( p_name, Obj_data )
char  p_name[];
ObjData		*Obj_data;
{
    FILE *fp;
    char buffer[130], vn_name[128], *bufptr;
    int i, j, lcount, nt, nn, value, vnct;
    int numNorms;
    Fcoord3D *norms;
    unsigned short *vn_indices;

    strcpy(vn_name, p_name);
    strcat(vn_name, ".vnorm");

    Obj_data->numNorms = 0;
    if((fp= fopen(vn_name, "r")) == NULL)  {
	return 0;
    }

    lcount = 0;
    numNorms = 0;
    norms = Obj_data->vertn;
    vn_indices = Obj_data->vn_indices;
    fscanf( fp, "%d %d %d\n",&vnct, &nt, &nn);
    /* norms = (pexCoord3D *) malloc( sizeof(pexCoord3D) * vnct); */

    for(i=0;i<vnct;i++) {
	fscanf( fp, "%f %f %f\n", &norms[i].x, &norms[i].y, &norms[i].z);
	    lcount++;
    }

    for(i=0;i<nt;i++) {
	    if(fgets(buffer, 128, fp) == NULL)
		break;
	    lcount++;

	    bufptr = strchr( buffer, '\n'); /* clear out the newline (\n) */
	    if( bufptr == NULL ) break;
	    *bufptr = 0;

	    sscanf(buffer,"%d",&j);	/* get number of indices */
	    bufptr = buffer;		/* look for first whitespace */
	    while( *bufptr != '\0' && !isspace(*bufptr)) bufptr++;
	    if( *bufptr == '\0' ) break;

	    for(; j>0; j--){
		/* look past spaces */
		while( *bufptr != '\0' &&  isspace(*bufptr) ) bufptr++;
		if( *bufptr == '\0' ) break;

		if( isdigit(*bufptr) )
		    value = atoi(bufptr);
		else {
		    error_index("polygon","normal",lcount);
		    break;}

		if( value > vnct+1 || value < 0 )
		    error_index("polygon","normal",lcount);

		vn_indices[numNorms] = value; /* 'value' is zero based */
		numNorms++;

		/* look for next non-digit */
	        while( *bufptr != '\0' && isdigit(*bufptr)) bufptr++;
		if( *bufptr == '\0' ) break;
	    }

    }

    fclose(fp);
    Obj_data->vnct = vnct;
    Obj_data->numNorms = numNorms;
    return vnct;
	
}

int
read_ipcol( p_name, Obj_data )
char  p_name[];
ObjData		*Obj_data;
{
    FILE *fp;
    char buffer[130], c_name[128], *bufptr;
    int i, j, lcount, nt, nn;
    int pcct;
    rgbFloat *pcol;
    unsigned short *vc_indices;

    strcpy(c_name, p_name);
    strcat(c_name, ".ipcol");

    Obj_data->pcct = 0;
    if((fp= fopen(c_name, "r")) == NULL)  {
	return 0;
    }

    lcount = 1;

    fscanf( fp, "%d %d\n",&pcct, &nt);

    if((Obj_data->pcol = (rgbFloat *) malloc( sizeof(int) * pcct)) == NULL )
	return 0;;
    if((Obj_data->vc_indices = (unsigned short *) 
	malloc( sizeof(short) * nt)) == NULL)
	return 0;
    pcol = Obj_data->pcol;
    vc_indices = Obj_data->vc_indices;
    Obj_data->pcct = pcct;

    for(i=0;i<pcct;i++) {
	fscanf( fp, "%f %f %f\n", &pcol[i].red, &pcol[i].green, &pcol[i].blue);
	    lcount++;
    }
    for(i=0;i<nt;i++) {
	fscanf( fp, "%d\n",&j);
	if( j > 0 )
	    vc_indices[i] = j - 1;
	else
	    vc_indices[i] = 0;
	lcount++;
	j = vc_indices[i];
	Obj_data->r = pcol[j].red;
	Obj_data->g = pcol[j].green;
	Obj_data->b = pcol[j].blue;
	DB_update_element( i+1, NULL, DB_PCLR_MASK, Obj_data);
    }

    fclose(fp);
    return nt;
	
}

int
read_pcol( p_name, Obj_data )
char  p_name[];
ObjData		*Obj_data;
{
    FILE *fp;
    char buffer[130], c_name[128], *bufptr;
    int i, j, lcount, nt, nn;
    int pcct;
    rgbFloat *pcol;

    strcpy(c_name, p_name);
    strcat(c_name, ".ipcol");

    Obj_data->pcct = 0;
    if((fp= fopen(c_name, "r")) == NULL)  {
	return 0;
    }

    lcount = 1;

    fscanf( fp, "%d\n",&pcct);

    if((Obj_data->pcol = (rgbFloat *) malloc( sizeof(int) * pcct)) == NULL )
	return 0;
    pcol = Obj_data->pcol;
    Obj_data->pcct = pcct;

    for(i=0;i<pcct;i++) {
	fscanf( fp, "%f %f %f\n", &pcol[i].red, &pcol[i].green, &pcol[i].blue);
	    lcount++;
	Obj_data->r = pcol[i].red;
	Obj_data->g = pcol[i].green;
	Obj_data->b = pcol[i].blue;
	DB_update_element( i+1, NULL, DB_PCLR_MASK, Obj_data);
    }

    fclose(fp);
    return pcct;
	
}

int
read_vcol( p_name, Obj_data )
char  p_name[];
ObjData		*Obj_data;
{
    FILE *fp;
    char buffer[130], c_name[128], *bufptr;
    int i, j, lcount;
    int vcct;
    rgbFloat *vcol;
    unsigned short *vc_indices;
    float *v_ptr;
    /* VERT_INDEX *vert_ind; */
    /* ELEMENTS *el_ptr; */

    strcpy(c_name, p_name);
    strcat(c_name, ".vcol");

    Obj_data->vcct = 0;
    if((fp= fopen(c_name, "r")) == NULL)  {
	return 0;
    }

    fscanf( fp, "%d\n",&vcct);
    lcount = 1;

    if((Obj_data->vcol = (rgbFloat *) malloc( sizeof(rgbFloat)*vcct)) == NULL )
	return 0;
    if((Obj_data->vc_indices = (unsigned short *) malloc( sizeof(short)*vcct)) 
	== NULL )
	return 0;
    Obj_data->vcct = vcct;
    vc_indices = Obj_data->vc_indices;
    vcol = Obj_data->vcol;

    for(i=0;i<vcct;i++) {
	fscanf( fp, "%f %f %f\n", 
		    &vcol[i].red, &vcol[i].green, &vcol[i].blue);
	vc_indices[i] = i;
    }

    fclose(fp);
    return vcct;
	
}

/*
 *  ROUTINE:	init_read_data
 *  PURPOSE:	Initializes a data file ( .pcol, .vcol ) for reading.
 *		The first value in the file must be the number of records.
 */
FILE *
init_read_data( p_name, Obj_data, suffix, value )
char  p_name[];
ObjData		*Obj_data;
char  *suffix;
int   *value;
{
    FILE *fp;
    char buffer[130], c_name[128], *bufptr;

    strcpy(c_name, p_name);
    strcat(c_name, suffix );

    if((fp= fopen(c_name, "r")) == NULL)  {
	return NULL;
    }
    fscanf( fp, "%d\n",value);
    return fp;
}

int
read_line_pcol( fp, Obj_data, i )
FILE  *fp;
ObjData		*Obj_data;
int	i;
{
    int j, lcount, nt, nn;
    rgbFloat *pcol;

    pcol = Obj_data->pcol;

	fscanf( fp, "%f %f %f\n", &pcol[i].red, &pcol[i].green, &pcol[i].blue);
	    lcount++;
	Obj_data->r = pcol[i].red;
	Obj_data->g = pcol[i].green;
	Obj_data->b = pcol[i].blue;

    Obj_data->mask |= DB_PCLR_MASK;
    Obj_data->pcct = Obj_data->pcct + 1;
}

int
read_aoff( p_name, Obj_data )
char  p_name[];
ObjData		*Obj_data;
{
    FILE *fp;
    char buffer[130], vn_name[128], *bufptr;
    int i, n, lcount, value;

    strcpy(vn_name, p_name);
    strcat(vn_name, ".aoff");

    if((fp= fopen(vn_name, "r")) == NULL)  {
	return 0;
    }

    while(1) {
	    if(fgets(buffer, 128, fp) == NULL)
		break;
	    lcount++;

	    bufptr = buffer;		/* look for first whitespace */
	    if( strncmp(buffer,"vertex_order",12) == 0 ){
		/*
		 *  go to end of buffer, back up to beginning of word, and
		 *  check what the ordering is
		 */
		n = strlen(buffer);
		i = n;
		i--; /* backup from new-line */
			/* go to last character */
		while( (i>0) && isspace(buffer[i]) )i--;	
			/* go to preceding space */
		while( (i>0) && !isspace(buffer[i]) )i--;
		i++;	/* move forward to first char */

		if( strncmp(&buffer[i],"counter",7) == 0){
			Obj_data->flip_flag = 0;
			break;
		}
		if( strncmp(&buffer[i],"clockwise",9) == 0){
			Obj_data->flip_flag = 1;
			break;
		}
	    }
    }

    fclose(fp);
    return 0;
	
}
